/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.clip;

import java.awt.*;
import java.awt.datatransfer.*;
import java.io.*;

import org.apache.log4j.Logger;
import com.notuvy.util.ValueHolder;

/**
 * A utility that provides read/write access to the system Clipboard.
 *
 * Notice that if this is loaded in a headless environment, then a stub implementation is
 * created.  This should never be a problem because the only meaningful headless environment
 * where this will be used is the unit test framework.
 * This utility (and the system clipboard) is only meaningful in a graphical environment.
 *
 * @author  murali
 */
public abstract class ClipboardAccess implements ClipboardOwner {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ClipboardAccess.class);

    private static ClipboardAccess INSTANCE = GraphicsEnvironment.isHeadless() ? new Dummy() : new GuiSystem();


    //------------------------------------------------------------
    //- Class Functions

    /**
     * For unit tests.
     * @return  The internal string value holder that the tests can use to simulate system clipboard changes.
     */
    protected static ValueHolder<String> setDummy() {
        Dummy dummy = new Dummy();
        INSTANCE = dummy;
        return dummy.fText;
    }


    public static String read() {
        return INSTANCE.readText();
    }


    public static void save(String pString) {
        INSTANCE.saveText(pString);
    }


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public abstract String readText();

    public abstract void saveText(String pString);


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public void lostOwnership(Clipboard pClipboard, Transferable pTransferable) {
        // This most likely happens when a another application places new content
        // in the system clipboard.  Checking by interrupting the likely waiting
        // thread which monitors contents.  This way, the polling behaves more like
        // an instantaneous call-back.
        ClipboardCallback.interrupt();
    }


    //------------------------------------------------------------
    //- Inner Classes

    static class GuiSystem extends ClipboardAccess {

        private final Clipboard fClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();


        @Override
        public String readText() {
            String result = "";

            //odd: the Object param of getContents is not currently used
            Transferable contents = fClipboard.getContents(this);
            boolean hasTransferableText =
                (contents != null) &&
                contents.isDataFlavorSupported(DataFlavor.stringFlavor);
            if ( hasTransferableText ) {
                try {
                    result = (String)contents.getTransferData(DataFlavor.stringFlavor);
                    if (result == null) {
                        result = "";
                    }
                } catch (UnsupportedFlavorException ex){
                    //highly unlikely since we are using a standard DataFlavor
                    LOG.error("Unsupported", ex);
                } catch (IOException ex) {
                    LOG.debug("Basic problem", ex);
                }
            }

            return result;
        }

        @Override
        public void saveText(String pString) {
            StringSelection selection = new StringSelection(pString);
            fClipboard.setContents(selection, this);
            ClipboardCallback.interrupt();
        }

    }

    static class Dummy extends ClipboardAccess {

        private ValueHolder<String> fText = new ValueHolder<String>("");

        @Override
        public String readText() {
            return fText.get();
        }

        @Override
        public void saveText(String pString) {
            fText.set(pString);
            ClipboardCallback.interrupt();
        }

    }


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            System.out.println(String.format("clipboard[%s]", ClipboardAccess.read()));
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
