/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.clip;

import java.util.*;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.log4j.Logger;

/**
 * An entity that monitors the system clipboard and notifies observers when
 * the contents have changed.
 *
 * @author  murali
 */
public class ClipboardCallback implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ClipboardCallback.class);

    public static final int SLEEP_TIME = 500;

    private static ClipboardCallback INSTANCE = new ClipboardCallback();


    //------------------------------------------------------------
    //- Class Functions

    public static void register(ClipboardListener pListener) {
        INSTANCE.addListener(pListener);
    }

    public static void deregister(ClipboardListener pListener) {
        INSTANCE.removeListener(pListener);
    }

    public static long countUnhandledExceptions() {
        return INSTANCE.fUnhandledExceptionCount.get();
    }

    protected static void interrupt() {
        INSTANCE.fThread.interrupt();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private String fLastContent = "";
    private final Collection<ClipboardListener> fListeners = new ArrayList<ClipboardListener>();
    private final AtomicLong fUnhandledExceptionCount = new AtomicLong();
    private final Thread fThread = new Thread(this, "ClipboardCallback");


    //------------------------------------------------------------
    //- Constructors

    private ClipboardCallback() {
        fThread.setDaemon(true);
        fThread.start();
    }


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected synchronized void addListener(ClipboardListener pListener) {
        fListeners.add(pListener);
    }

    protected synchronized void removeListener(ClipboardListener pListener) {
        fListeners.remove(pListener);
    }


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        //noinspection InfiniteLoopStatement
        while (true) {
            try {
                String current = ClipboardAccess.read();
                if (current.equals(fLastContent)) {
                    try {
                        Thread.sleep(SLEEP_TIME);
                    } catch (InterruptedException ie) {
                        // Not an error.  Someone wants this to run again immediately.
                    }
                } else {
                    synchronized (this) {
                        for (ClipboardListener clipboardListener : fListeners) {
                            clipboardListener.newClipboardContents(current);
                        }
                    }

                    fLastContent = current;
                }
            } catch (Throwable thr) {
                LOG.error("Unhandled error in clipboard thread.", thr);
                fUnhandledExceptionCount.incrementAndGet();
            }
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
