/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import org.apache.commons.lang.StringUtils;

import java.util.*;

import com.notuvy.util.Translation;
import com.notuvy.util.ApplicationProperties;

/**
 * A command line argument interpreter.
 *
 * User: murali
 * Date: Jan 2, 2008
 * Time: 2:04:45 PM
 */
public class CommandSet extends CommandAction {

    //------------------------------------------------------------
    //- Class Variables


    public static final String CMD_HELP = "help";
    public static final String CMD_VERSION = "version";

    private static final Comparator<String> NORMAL_ORDER_WITH_HELP_VERSION_PRIORITIZED = new Comparator<String>() {
        public int compare(String s1, String s2) {
            int result;
            if (CMD_HELP.equals(s1)) {
                result = -1;
            } else if (CMD_HELP.equals(s2)) {
                result = 1;
            } else if (CMD_VERSION.equals(s1)) {
                result = -1;
            } else if (CMD_VERSION.equals(s2)) {
                result = 1;
            } else {
                result = s1.compareTo(s2);
            }
            return result;
        }
    };


    //------------------------------------------------------------
    //- Class Functions

    /**
     * The top-level entry point to application execution.
     * Create a new CommandSet, have the application configure it, process the
     * arguments, then run the application.
     *
     * @param pCommandable  The initialized application instance.
     * @param pArgs  The raw command line args.
     */
    public static void execute(Commandable pCommandable, String[] pArgs) {
        try {
            CommandSet cmdset = new CommandSet();

            pCommandable.configure(cmdset);

            if (cmdset.process(pArgs) && !cmdset.isTerminated()) {
                pCommandable.run();
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }

    public static void execute(DefaultedCommandable pCommandable, String[] pArgs) {
        try {
            CommandSet cmdset = new CommandSet();

            if (pArgs.length == 0) {
            } else {
                CommandArgs args = new CommandArgs(pArgs);
                pCommandable.configure(args);
                pCommandable.run();
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }

    public static void execute(SimpleExecutable pExecutable, String[] pArgs) {
        try {
            if (pExecutable.setup(new CommandArgs(pArgs))) {
                pExecutable.run();
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }

    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String,CommandAction> fMap = new HashMap<String,CommandAction>();
    private PrefixSynonymMap fPrefixSynonyms = null;
    private final List<String> fHandled = new ArrayList<String>();
    private final Translation<String> fTranslation = new Translation.Rewrite();
    private boolean fAllowEmpty = false;

    //------------------------------------------------------------
    //- Constructors

    public CommandSet() {
        this (null);

        define(new Command(CMD_HELP) {
            public boolean process(CommandArgs pArgs) {
                LOG.info(getParent().toUsageString());
                setTerminated(true);
                return true;
            }
        });
        defineSynonym("-"+CMD_HELP, CMD_HELP);
    }

    // Here for subclass usage (CommandSubSet).
    protected CommandSet(String pKey) {
        super (pKey);
    }

    //------------------------------------------------------------
    //- Getters

    protected Map<String, CommandAction> getMap() {
        return fMap;
    }

    private PrefixSynonymMap getPrefixSynonyms() {
        // This needs to be done with lazy evaluation.  All the keys must be defined
        // in order to find the full set of unambiguous prefixes.
        if (fPrefixSynonyms == null) {
            Set<String> all = new HashSet<String>();
            all.addAll(getMap().keySet());
            all.addAll(getTranslation().getUnderlying().keySet());
            fPrefixSynonyms = new PrefixSynonymMap(all);
        }
        return fPrefixSynonyms;
    }

    private Translation<String> getTranslation() {
        return fTranslation;
    }

    /**
     * Give the list of all commands that were handled.
     * 
     * @return The list of String names of executed commands.
     */
    public List<String> getHandled() {
        return fHandled;
    }

    /**
     * Whether it is acceptable for the command set to be executed with no arguments.
     * @return boolean flag.
     */
    public boolean isAllowEmpty() {
        return fAllowEmpty;
    }

    //------------------------------------------------------------
    //- Settors

    /**
     * Define whether it is acceptable for the command set to be executed with no
     * arguments.  Some usages will require that an action must be specified.  Others
     * use the command interpretor just to define settings (and none may be specified)
     * and the actual action may be triggered externally.
     *
     * @param pAllowEmpty  boolean flag.
     */
    public void setAllowEmpty(boolean pAllowEmpty) {
        fAllowEmpty = pAllowEmpty;
    }

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected boolean processNext(CommandArgs pArgs) {
        boolean valid = false;
        String command = pArgs.next();
        CommandAction action = lookupCommand(command);
        if (action == null) {
            LOG.error("Unrecognized command [" + command + "]");
            LOG.error(toUsageString());
        } else {
            valid = action.handle(pArgs);
            if (valid) {
                getHandled().add(action.getKey());
                if (action.isTerminated()) {
                    setTerminated(true);
                }
            }
            // todo: fix the usage printing of leaves only.
//            if (!valid) {
//                LOG.error(action.toUsageString());
//            }
        }
        return valid;
    }

    private CommandAction lookupCommand(String pGiven) {
        CommandAction result = getMap().get(getTranslation().lookup(pGiven));

        if (result == null) {
            // If there was not an exact match to a command, check if
            // we received a synonym of the command.
            // Convert synonym to full command and re-lookup.
            String full = getPrefixSynonyms().lookup(pGiven);
            if (full != null) {
                result = getMap().get(getTranslation().lookup(full));
            }
        }
        return result;
    }

    public void defineVersionFrom(final Class pReferenceClass) {
        defineVersionFrom(pReferenceClass, "");
    }

    public void defineVersionFrom(final Class pReferenceClass, final String pAdditional) {
        define(new Command(CMD_VERSION) {
            public boolean process(CommandArgs pArgs) {
                ApplicationProperties mp = ApplicationProperties.from(pReferenceClass);
                StringBuilder builder = new StringBuilder("\n");
                if (StringUtils.isNotBlank(pAdditional)) {
                    builder.append(pAdditional).append("\n");
                }
                builder.append(mp).append("\n");
                builder.append(mp.toCopyrightString()).append("\n");
                if (!ApplicationProperties.UNDEFINED.equals(mp.getProjectDescription())) {
                    builder.append(mp.getProjectDescription()).append("\n");
                }
                LOG.info(builder);
                setTerminated(true);
                return true;
            }
        });
        defineSynonym("-"+CMD_VERSION, CMD_VERSION);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * Add the given command to this set.
     *
     * @param pAction action object
     * @return this, for cascading.
     */
    public CommandSet define(CommandAction pAction) {
        String key = pAction.getKey();
        if (StringUtils.isBlank(key)) {
            LOG.error("No key for command.");
        } else {
            pAction.setParent(this);
            getMap().put(key, pAction);
        }
        return this;
    }

    /**
     * Define a command as a synonym for an existing command.  (Duplication without redefinition.)
     *
     * @param pSynonym  The new command to define.
     * @param pKey  The name of the existing command to duplicate.
     * @return this, for cascading.
     */
    public CommandSet defineSynonym(String pSynonym, String pKey) {
        getTranslation().define(pSynonym, pKey);
        return this;
    }

    /**
     * The primary action is defined externally by the given Runnable, so process all the
     * given arguments, then execute the runnable (assuming no errors during processing).
     *
     * @param pArgs  The list of arguments (from the main method).
     * @param pRunnable  The externally define principle action.
     */
    public void processAndExecute(String[] pArgs, Runnable pRunnable) {
        if (process(pArgs) && !isTerminated()) {
            pRunnable.run();
        }
    }

    public String toString() {
        StringBuilder builder = new StringBuilder("Usage:\n");
        for (String line : listCommands()) {
            builder.append("  ").append(line).append("\n");
        }
        return builder.toString();
    }

    //------------------------------------------------------------
    //- Class Interface Functions


    protected boolean handle(CommandArgs pArgs) {
        boolean valid = true;
        getHandled().clear();

        if (pArgs.isEmpty()) {
            if (!isAllowEmpty()) {
                LOG.info(this);  // If no arguments, print the usage.
                valid = false;
            }
        } else {
            // Need to loop here.  We want to allow multiple property switches
            // to be set.  If any commands need to be terminal, the can set
            // the flag in their implementation.
            while (valid && !pArgs.isEmpty()) {
                valid = processNext(pArgs);
                if (valid && isTerminated()) {
                    valid = pArgs.exhausted();
                }
            }
        }


        return valid;
    }


    public List<String> listCommands() {
        List<String> result = preliminaryCommands();
        Collections.sort(result, NORMAL_ORDER_WITH_HELP_VERSION_PRIORITIZED);

        return result;
    }

    protected List<String> preliminaryCommands() {
        List<String> preliminary = new ArrayList<String>();
        for (CommandAction commandAction : getMap().values()) {
            preliminary.addAll(commandAction.listCommands());
        }
        return preliminary;
    }

    public String toUsageString() {
        return toString();
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
