/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.cmd;

import java.util.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Create unique synonyms for a given set of commands.
 * A full command is broken into all its prefixes (i.e. "load" becomes
 * "loa", "lo", "l".  All of these which do not conflict with any
 * others are kept as synonyms.
 *
 * User: murali
 * Date: Jan 3, 2008
 * Time: 8:00:21 AM
 */
public class PrefixSynonymMap {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Pattern P_MULTIPLE = Pattern.compile("^([\\-\\w]+)\\w$");

    //------------------------------------------------------------
    //- Class Functions

    /**
     * Determine the full set of prefixes that can be extracted from the given word.
     *
     * @param pInput Input command.
     * @return  The full collection of prefix strings.
     */
    protected static Collection<String> allPrefixes(String pInput) {
        String current = pInput;
        ArrayList<String> result = new ArrayList<String>();
        Matcher matcher = P_MULTIPLE.matcher(current);
        while (matcher.find()) {
            current = matcher.replaceFirst("$1");
            result.add(current);
            matcher = P_MULTIPLE.matcher(current);
        }
        Collections.sort(result);
        return result;
    }

    protected static Collection<String> selectAll(Pattern pPattern, Collection<String> pStrings) {
        ArrayList<String> result = new ArrayList<String>();
        for (String string : pStrings) {
            if (pPattern.matcher(string).find()) {
                result.add(string);
            }
        }
        return result;
    }

    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String,String> fMap = new HashMap<String,String>();


    //------------------------------------------------------------
    //- Constructors

    /**
     * Note that the full set of commands must be given on creation.  We cannot
     * allow iterative addition because the full set must be known to accurately
     * compute the unique prefixes.
     *
     * @param pOriginals The full set of strings to process.
     */
    public PrefixSynonymMap(Set<String> pOriginals) {
        for (String original : pOriginals) {
            Collection<String> prefixes = allPrefixes(original);
            for (String prefix : prefixes) {
                // Now check how many of the original strings match the prefix.  Only
                // keep the unique prefixes.
                Pattern prefixPattern = Pattern.compile("^" + prefix + "\\w*$");
                Collection<String> matches = selectAll(prefixPattern, pOriginals);
                if (matches.size() == 1) {
                    // If the collection has only one, then it must be just the original.
                    getMap().put(prefix, original);
                }
            }
        }
    }

    //------------------------------------------------------------
    //- Getters

    protected Map<String, String> getMap() {
        return fMap;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public String lookup(String pKey) {
        return getMap().get(pKey);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
