package com.notuvy.condtrig;

import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

/**
 * Abstract representation of the antecedant of the trigger.
 * It could be a single value, or could be a complex composite.
 *
 * If a leave value is modified, the final result value is recomputed by recalculating the value
 * of the branch from the root to that leaf.
 * If the final result value changes, then the consequent is triggered with that new value.
 *
 * Note that all sub-expressions can also have their own local consequents.
 *
 * @author murali
 */
public abstract class Expression {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.snipfolio.condtrig");

    private static final Expression ROOT = new Expression() {
        protected void recalculate() {
            // Nothing
        }

        @Override
        protected void initializeNewValue(boolean pNewValue) {
        }

        @Override
        public String toString() {
            return "ROOT";
        }
    };


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private Expression fParent = ROOT;
    private final List<Consequent> fConsequents = new ArrayList<Consequent>();
    protected boolean fValue = false;


    //------------------------------------------------------------
    //- Constructors

    protected Expression() {
        this (false);
    }

    protected Expression(boolean pValue) {
        initializeNewValue(pValue);
    }


    //------------------------------------------------------------
    //- Gettors

    public Expression getParent() {
        return fParent;
    }

    public boolean getValue() {
        return fValue;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setParent(Expression pParent) {
        fParent = pParent;
    }

    protected void setValue(boolean pNewValue) {
        if (pNewValue != getValue()) {
            initializeNewValue(pNewValue);
        }
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void initializeNewValue(boolean pNewValue) {
        fValue = pNewValue;

        for (Consequent consequent : fConsequents) {
            consequent.handle(pNewValue);
        }
        getParent().recalculate();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public Expression addConsequent(Consequent pConsequent) {
        fConsequents.add(pConsequent);

        // process the initial state
        pConsequent.handle(getValue());

        return this;
    }

    public void removeConsequent(Consequent pConsequent) {
        fConsequents.remove(pConsequent);
    }

    public void clearConsequents() {
        fConsequents.clear();
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected abstract void recalculate();


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
