/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import org.apache.log4j.Logger;

import java.io.*;
import java.util.*;

/**
 * A utility for managing directories.
 *
 * @author  murali
 * @version  $Id: Directory.java,v 1.5 2006/06/20 04:52:41 murali Exp $
 */
public class Directory {

    //------------------------------------------------------------
    //- Class Variables

    public static final Directory HOME = new Directory(System.getProperty("user.home"));
    public static final String APP_HOME_PROPERTY = "app.home";

    // Do not create a static Logger.  There will be a circular dependency.

    private static final FileFilter PLAIN_FILES = new FileFilter() {
            public boolean accept(File pFile) {
                return pFile.isFile();
            }
        };

    //------------------------------------------------------------
    //- Class Functions

    public static Directory ensure(String pFilename) {
        Directory dir = new Directory(pFilename);
        dir.ensure();
        return dir;
    }


    public static Directory ensure(File pFile) {
        Directory dir = new Directory(pFile);
        dir.ensure();
        return dir;
    }


    public static Directory ensureInHome(String pFilename) {
        return HOME.subdir(pFilename);
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final File vFile;


    //------------------------------------------------------------
    //- Constructors

    public Directory(File pFile) {
        vFile = pFile;
    }

    public Directory(String pString) {
        this (new File(pString));
    }



    //------------------------------------------------------------
    //- Accessors

    public File getFile() {
        return vFile;
    }



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions



    //------------------------------------------------------------
    //- Public Interface Functions

    public void ensure() {
        Directory parent = new Directory(getFile().getParentFile());
        if (!parent.getFile().exists()) {
            parent.ensure();
        }

        if (!getFile().exists()) {
            if (getFile().mkdir()) {
                if (System.getProperty(APP_HOME_PROPERTY) != null) {  // Do not try to access log4j if not initialized.
                    Logger.getLogger(getClass()).info("Created dir [" + getFile() + "].");
                }
            }
        }
    }


    public File entry(String pFilename) {
        return new File(getFile(), pFilename);
    }


    public Directory subdir(String pFilename) {
        File file = new File(getFile(), pFilename);
        Directory result = new Directory(file);
        result.ensure();
        return result;
    }

    public Directory setAppHome() {
        System.setProperty(APP_HOME_PROPERTY, getFile().getAbsolutePath());
        return this;
    }

    /**
     * Find all the (first level) contents of this directory.
     * @return Unordered list of files, potentially a mix of plain files and subdirectories.
     */
    public List<File> listContents() {
        return new ArrayList<File>(Arrays.asList(getFile().listFiles()));
    }


    /**
     * Find all the full depth contents of this directory.  Follow all subdirectories.
     * That is, do a depth-first search for all plain files.
     * @return Unordered list of plain files.
     */
    public List<File> listDeepContents() {
        List<File> result = new ArrayList<File>();

        for (File file : getFile().listFiles()) {
            if (file.isDirectory()) {
                Directory parent = new Directory(file);
                for (File descendant : parent.listDeepContents()) {
                    result.add(descendant);
                }
            } else {
                result.add(file);
            }
        }

        return result;
    }

    /**
     * Create the structure containing the contents of this directory, with
     * plain files and subdirs segregated.
     * @return  contents in a structured object.
     */
    public DirectoryContents contents() {
        return new DirectoryContents(this);
    }

    public List<File> listContents(FilenameFilter pFilter) {
        return new ArrayList<File>(Arrays.asList(getFile().listFiles(pFilter)));
    }


    public boolean isEmpty() {
        return (getFile().listFiles().length == 0);
    }


    public void clear() {
        for (File file : getFile().listFiles(PLAIN_FILES)) {
            if (file.delete()) {
                if (System.getProperty(APP_HOME_PROPERTY) != null) {  // Do not try to access log4j if not initialized.
                    Logger.getLogger(getClass()).debug("deleted [" + file + "].");
                }
            }
        }
    }


    public Collection<File> findFileInSubdir(String pFilename) {
        Collection<File> result = new ArrayList<File>();

        for (File subdir : listContents()) {
            if (subdir.isDirectory()) {
                File candidate = new File(subdir, pFilename);
                if (candidate.exists() && candidate.isFile()) {
                    result.add(candidate);
                }
            }
        }
        return result;
    }

    public String relativeNameOf(File pFile) {
        List<String> parts = new ArrayList<String>();
        parts.add(pFile.getName());
        File candidate = pFile.getParentFile();
        while ((candidate != null) && !candidate.equals(getFile())) {
            parts.add(File.separator);
            parts.add(candidate.getName());
            candidate = candidate.getParentFile();
        }

        Collections.reverse(parts);
        StringBuffer buffer = new StringBuffer();
        for (String part : parts) {
            buffer.append(part);
        }
        return buffer.toString();
    }

    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main


}
