/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import java.io.*;
import java.util.*;

import org.apache.log4j.Logger;

/**
 * A utility to copy files.
 *
 * @author  murali
 * @version  $Id: Sample.java,v 1.1 2006/06/07 03:29:59 murali Exp $
 */
public class FileCompare implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileCompare.class);

    private static final int BUFFER_SIZE = 1024;


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final File vSource;
    private final File vDestination;
    private boolean vFilesAreSame = false;
    private boolean vUncompared = true;


    //------------------------------------------------------------
    //- Constructors

    public FileCompare(File pSource, File pDestination) {
        vSource = pSource;

        vDestination = pDestination.isDirectory() ?
            new File(pDestination, pSource.getName()) :
            pDestination;
    }


    //------------------------------------------------------------
    //- Accessors

    public File getSource() {
        return vSource;
    }

    public File getDestination() {
        return vDestination;
    }

    protected boolean getFilesAreSame() {
        return vFilesAreSame;
    }

    protected boolean getUncompared() {
        return vUncompared;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setFilesAreSame(boolean pFilesAreSame) {
        vFilesAreSame = pFilesAreSame;
    }

    protected void setUncompared(boolean pUncompared) {
        vUncompared = pUncompared;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void doComparison() throws IOException {
        boolean same = true;  // Optimistic.
        boolean dataRemaining = true;

        byte[] buffer1 = new byte[BUFFER_SIZE];
        FileInputStream fis1 = new FileInputStream(getSource());
        BufferedInputStream reader1 = new BufferedInputStream(fis1);

        byte[] buffer2 = new byte[BUFFER_SIZE];
        FileInputStream fis2 = new FileInputStream(getDestination());
        BufferedInputStream reader2 = new BufferedInputStream(fis2);

        long total = 0;

        while (dataRemaining && same) {
            int bytesRead1 = reader1.read(buffer1, 0, BUFFER_SIZE);
            // LOG.debug("read " + bytesRead1 + " bytes from file1.");
            int bytesRead2 = reader2.read(buffer2, 0, BUFFER_SIZE);
            // LOG.debug("read " + bytesRead2 + " bytes from file2.");
            if (bytesRead1 == bytesRead2) {
                total += bytesRead1;
                if (bytesRead1 >= 0) {
                    for (int i = 0; same && (i < bytesRead1); i++) {
                        same = (buffer1[i] == buffer2[i]);
                        if (!same) {
                            // LOG.debug("Differ [" + ((int)buffer1[i]) + "] [" + ((int)buffer2[i]) + "], at [" + (total+i) + "].");
                        }
                    }
                } else {
                    dataRemaining = false;
                }
            } else {
                LOG.error("Incompatible block sizes.");
                same = false;
            }
        }

        setFilesAreSame(same);

        reader1.close();
        fis1.close();
        reader2.close();
        fis2.close();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean isSame() {
        run();

        boolean result = getFilesAreSame();
        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        try {
            if (getUncompared()) {
                setUncompared(false);
                LOG.debug("Compare [" + getSource() + "] to [" + getDestination() + "].");
                if (!getSource().exists()) {
                    LOG.error("[" + getSource() + "] does not exist.");
                } else if (!getDestination().exists()) {
                    LOG.error("[" + getDestination() + "] does not exist.");
                } else if (getSource().length() != getDestination().length()) {
                    LOG.debug("Files are different sizes.");
                } else {
                    doComparison();
                }
            }
        } catch (IOException ioe) {
            LOG.error("Problem comparing files.", ioe);
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
