/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.file;

import java.io.*;
import java.util.*;

import org.apache.log4j.Logger;

/**
 * A utility to save string content to a file.
 *
 * @author  murali
 */
public class FileContent {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileContent.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final File vDestination;


    //------------------------------------------------------------
    //- Constructors

    public FileContent(File pParent, String pName) {
        vDestination = new File(pParent, pName);
    }


    public FileContent(File pDestination) {
        vDestination = pDestination;
    }


    //------------------------------------------------------------
    //- Accessors

    public File getDestination() {
        return vDestination;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean fileExists() {
        boolean result = getDestination().exists();
        return result;
    }


    public boolean save(Object pContent) {
        boolean success = false;  // Pessimistic.
        try {
            FileWriter writer = new FileWriter(getDestination());
            writer.write(String.valueOf(pContent));
            writer.close();
            // LOG.info("Wrote [" + getDestination() + "]");
            success = true;
        } catch (FileNotFoundException fnfe) {
            LOG.error("No output file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem writing output file [" + getDestination() + "].", ioe);
        }

        return success;
    }


    public String readString() {
        StringWriter writer = new StringWriter();

        try {
            transferTo(writer);
            writer.close();
        } catch (IOException ioe) {
            LOG.error("Problem reading string output file [" + getDestination() + "].", ioe);
        }

        return writer.toString();
    }


    public void transferTo(Writer pWriter) throws IOException {
        try {
            char[] chunk = new char[1024];

            FileReader reader = new FileReader(getDestination());
            for (int numRead=reader.read(chunk); numRead!=-1; numRead=reader.read(chunk)) {
                pWriter.write(chunk, 0, numRead);
            }
            reader.close();
            LOG.debug("Read [" + getDestination() + "]");
        } catch (FileNotFoundException fnfe) {
            LOG.error("No input file.", fnfe);
        }
    }


    public boolean readLines(LineHandler pHandler) {
        boolean success = false;  // Pessimistic.

        try {
            FileReader fr = new FileReader(getDestination());
            LineNumberReader lnr = new LineNumberReader(fr);

            for (String line=lnr.readLine(); line!=null; line=lnr.readLine()) {
                pHandler.handle(lnr.getLineNumber(), line);
            }

            lnr.close();
            fr.close();
            success = true;
        } catch (FileNotFoundException fnfe) {
            LOG.error("No input file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem reading input file.", ioe);
        }

        return success;
    }


    public boolean save(WriterHandler pHandler) {
        boolean success = false;  // Pessimistic.

        try {
            FileWriter writer = new FileWriter(getDestination());

            pHandler.handle(writer);

            writer.close();

            success = true;
        } catch (IOException ioe) {
            LOG.error("Could not save account data to [" + getDestination() + "].", ioe);
        }

        return success;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes

    public interface LineHandler {
        public void handle(int number, String line);
    }

    public interface WriterHandler {
        public void handle(FileWriter writer) throws IOException;
    }



    //------------------------------------------------------------
    //- Main



}
