/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import java.io.*;

import org.apache.log4j.Logger;
import com.notuvy.util.Progressable;
import com.notuvy.util.DummyProgress;

/**
 * A utility to copy files.
 *
 * @author  murali
 * @version  $Id: Sample.java,v 1.1 2006/06/07 03:29:59 murali Exp $
 */
public class FileCopy implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileCopy.class);

    private static final int BUFFER_SIZE = 1024;


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private File vSource;
    private File vDestination;
    private boolean vSuccessful = false;
    private boolean vCreateDestinationParent = false;
    private boolean vNoOverwrite = false;
    private Progressable vProgress = new DummyProgress();


    //------------------------------------------------------------
    //- Constructors

    public FileCopy(File pSource, File pDestination) {
        vSource = pSource;

        vDestination = pDestination.isDirectory() ?
            new File(pDestination, pSource.getName()) :
            pDestination;
    }


    //------------------------------------------------------------
    //- Accessors

    public File getSource() {
        return vSource;
    }

    public File getDestination() {
        return vDestination;
    }

    public boolean getSuccessful() {
        return vSuccessful;
    }

    public boolean getCreateDestinationParent() {
        return vCreateDestinationParent;
    }

    public boolean getNoOverwrite() {
        return vNoOverwrite;
    }

    public Progressable getProgress() {
        return vProgress;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setSuccessful(boolean pSuccessful) {
        vSuccessful = pSuccessful;
    }

    public void setCreateDestinationParent(boolean pCreateDestinationParent) {
        vCreateDestinationParent = pCreateDestinationParent;
    }

    public void setNoOverwrite(boolean pNoOverwrite) {
        vNoOverwrite = pNoOverwrite;
    }

    public void setProgress(Progressable pProgress) {
        vProgress = pProgress;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    /**
     * Check all the preconditions for a valid copy.
     * If the parent directory of the destination does not exist, then create
     * if directed to do so, or fail if not.
     *
     * @return  A boolean flag that indicates that all the precondtions are valid.
     */
    protected boolean preconditionsHold() {
        boolean result = false;  // Pessimistic.

        if (getSource().exists()) {
            if (getSource().equals(getDestination())) {
                LOG.error("Cannot copy [" + getSource() + "] to itself.");
            } else {
                File parent = getDestination().getParentFile();

                if (getCreateDestinationParent()) {
                    Directory.ensure(parent);
                }

                if (parent.exists()) {
                    if (getDestination().exists() && getNoOverwrite()) {
                        LOG.error("No overwrite of [" + getDestination() + "]");
                    } else {
                        result = true;  // Okay to proceed.
                    }
                } else {
                    LOG.error("Target parent [" + parent + "] does not exist.");
                }
            }
        } else {
            LOG.error("Source file [" + getSource() + "] does not exist.");
        }

        return result;
    }


    protected void doCopy() throws IOException {
        try {
            getProgress().start();

            byte[] buffer = new byte[BUFFER_SIZE];
            FileInputStream fis = new FileInputStream(getSource());
            BufferedInputStream reader = new BufferedInputStream(fis);
            FileOutputStream fos = new FileOutputStream(getDestination());
            BufferedOutputStream writer = new BufferedOutputStream(fos);

            getProgress().setMaximum((int) getSource().length());

            for (int bytesRead=reader.read(buffer, 0, BUFFER_SIZE); bytesRead>=0; bytesRead=reader.read(buffer, 0, BUFFER_SIZE)) {
                getProgress().incrementValue(bytesRead);
                writer.write(buffer, 0, bytesRead);
            }

            reader.close();
            fis.close();
            writer.close();
            fos.close();
        } finally {
            getProgress().finished();
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        setSuccessful(false);
        try {
            if (preconditionsHold()) {
                doCopy();
                LOG.debug("Copied [" + getSource() + "] to [" + getDestination() + "].");
                setSuccessful(true);
            }
        } catch (IOException ioe) {
            LOG.error("Problem copying files.", ioe);
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
