/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import java.io.File;
import java.util.LinkedList;

import org.apache.log4j.Logger;

/**
 * Decompose the total bytes to larger denominations.
 *
 * @author  murali
 */
public class FileSize {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileSize.class);

    public static final int KILO = 1024;
    public static final int HALF_KILO = KILO / 2;
    public static final int MEGA = KILO * KILO;

    private static final String[] NAMES = {"bytes", "Kb", "Mb", "Gb"};


    //------------------------------------------------------------
    //- Class Functions

    public static long mbToBytes(long pBytes) {
        long result = pBytes * MEGA;
        return result;
    }


    public static long bytesToMb(long pBytes) {
        long remainder = pBytes % MEGA;

        long result = pBytes / MEGA;
        if (remainder > 0) {
            // Perform ceiling function.
            result++;
        }

        return result;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final long vTotal;
    private final LinkedList vParts = new LinkedList();


    //------------------------------------------------------------
    //- Constructors

    public FileSize(long pTotal) {
        vTotal = pTotal;

        int i = 0;
        do {
            long remainder = pTotal % KILO;
            getParts().addFirst(new Part(remainder, NAMES[i++]));

            pTotal = pTotal / KILO;
            if (remainder >= HALF_KILO) {
                pTotal += 1;
            }
        } while ((i < NAMES.length) && (pTotal != 0));

        if (pTotal != 0) {
            Part part = (Part) getParts().removeFirst();
            long number = (KILO * pTotal) + part.getNumber();
            getParts().addFirst(new Part(number, part.getName()));
        }
    }


    public FileSize(File pFile) {
        this (pFile.length());
    }


    //------------------------------------------------------------
    //- Accessors

    public long getTotal() {
        return vTotal;
    }

    protected LinkedList getParts() {
        return vParts;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        Part part = (Part) getParts().getFirst();
        String result = part.toString();
        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes

    class Part {

        private final long vNumber;
        private final String vName;


        protected Part(long pNumber, String pName) {
            vNumber = pNumber;
            vName = pName;
        }


        public long getNumber() {
            return vNumber;
        }

        public String getName() {
            return vName;
        }

        public String toString() {
            String result = getNumber() + " " + getName();
            return result;
        }

        public boolean shouldRoundNext() {
            boolean result = (getNumber() >= HALF_KILO);
            return result;
        }

        public int giveFirstDecimal() {
            int result = (int) getNumber() / 100;
            return result;
        }
    }


    //------------------------------------------------------------
    //- Main


}
