/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.file;

import org.apache.log4j.Logger;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.io.File;

/**
 * Safely create a file by finding a numbered varient of a given pattern.
 *
 * User: murali
 * Date: Aug 26, 2008
 * Time: 10:14:38 AM
 */
public class SafeFilename {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger gLogger = Logger.getLogger(SafeFilename.class);

    private static final Pattern P_TEMPLATE_NO_SUFFIX = Pattern.compile("^([^\\.]+)$");
    private static final Pattern P_TEMPLATE_WITH_SUFFIX = Pattern.compile("^(.+)\\.([^\\.]+)$");

    //------------------------------------------------------------
    //- Class Functions

    public static File get(File pFile) {
        SafeFilename generator = new SafeFilename(pFile);
        return generator.next();
    }

    public static File get(String pFilename) {
        return get(new File(pFilename));
    }

    //------------------------------------------------------------
    //- Instance Variables

    private final File fParent;
    private final String fOriginal;
    private final String fBase;
    private final String fSuffix;
    private int fIndex = 1;

    //------------------------------------------------------------
    //- Constructors

    public SafeFilename(File pTemplate) {
        fParent = pTemplate.getParentFile();
        fOriginal = pTemplate.getName();

        Parts parts = new Parts(pTemplate.getName());
        if (parts.fValid) {
            fBase = parts.fBase;
            fSuffix = parts.fSuffix;
        } else {
            fBase = "";
            fSuffix = "";
        }
    }

    //------------------------------------------------------------
    //- Getters

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private synchronized String nextName() {
        StringBuffer number = new StringBuffer();
        number.append(fIndex++);
        while (number.length() < 3) {
            number.insert(0, "0");
        }
        return fBase + "_" + number + "." + fSuffix;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public synchronized File next() {
        File candidate = (fIndex == 1) ? new File(fParent, fOriginal) : null;

        while ((candidate == null) || candidate.exists()) {
            String name = nextName();
            candidate = new File(fParent, name);
        }

        return candidate;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    public static class Parts {
        public String fBase = "";
        public String fSuffix = "";
        public boolean fValid = false;

        public Parts(String pString) {
            Matcher matcherNoSuffix = P_TEMPLATE_NO_SUFFIX.matcher(pString);
            Matcher matcherWithSuffix = P_TEMPLATE_WITH_SUFFIX.matcher(pString);
            if (matcherNoSuffix.find()) {
                fValid = true;
                fBase = matcherNoSuffix.group(1);
            } else if (matcherWithSuffix.find()) {
                fValid = true;
                fBase = matcherWithSuffix.group(1).replaceFirst("_$", "");
                fSuffix = matcherWithSuffix.group(2);
            }
        }
    }

    //------------------------------------------------------------
    //- Main

}
