package com.notuvy.gui;

import org.apache.log4j.Logger;

import javax.swing.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.annotation.IncompleteAnnotationException;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.List;

/**
 * A utility to attach action listeners to components.
 * It is instantiated with a reference to a GuiActionHandler that contains the methods
 * to invoke for the UI actions.
 * If a status bar is given, then the clear action is prepended to the action listener.
 *
 * @author murali
 */
public class ActionAttacher {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.gui");


    //------------------------------------------------------------
    //- Class Functions

    private static boolean classDeclaresGuiActionHandler(Class pClass) {
        boolean result = false;
        for (Class interfaceClass : pClass.getInterfaces()) {
            if (interfaceClass.equals(GuiActionHandler.class)) {
                result = true;
            }
        }
        return result;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final GuiActionHandler fHandler;
    private final Map<String,Method> fHandlerMethods = new HashMap<String,Method>();
    private StatusBar fStatusBar;


    //------------------------------------------------------------
    //- Constructors

    public ActionAttacher(GuiActionHandler pHandler) {
        fHandler = pHandler;

        extractLabeledMethods(pHandler.getClass());
    }


    //------------------------------------------------------------
    //- Gettors

    public StatusBar getStatusBar() {
        return fStatusBar;
    }


    //------------------------------------------------------------
    //- Settors

    public void setStatusBar(StatusBar pStatusBar) {
        fStatusBar = pStatusBar;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void extractLabeledMethods(Class pHandlerClass) {
        // Lookup ALL classes in the heirarchy that implement interface GuiActionHandler.
        List<Class> classes = new ArrayList<Class>();
        classes.add(pHandlerClass);
        if (!classDeclaresGuiActionHandler(pHandlerClass)) {
            Class candidate = pHandlerClass.getSuperclass();
            while (!classDeclaresGuiActionHandler(candidate)) {
                classes.add(candidate);
                candidate = candidate.getSuperclass();
            }
            classes.add(candidate);
        }

        for (Class aClass : classes) {
            for (Method method : aClass.getDeclaredMethods()) {
                GuiAction action = method.getAnnotation(GuiAction.class);
                if (action != null) {
                    fHandlerMethods.put(action.value(), method);
                }
            }
        }
    }

    protected ActionListener createActionListenerFor(String pLabel) {
        final Method method = fHandlerMethods.get(pLabel);
        if (method == null) {
            throw new IncompleteAnnotationException(GuiAction.class, String.format("Could not find method labeled [%s].", pLabel));
        } else {
            return new ActionListener() {
                public void actionPerformed(ActionEvent pActionEvent) {
                    try {
                        method.invoke(fHandler);
                    } catch (IllegalAccessException iae) {
                        LOG.error(String.format("Could not access method [%s].", method.getName()), iae);
                    } catch (InvocationTargetException ite) {
                        LOG.error(String.format("Error invoking method [%s].", method.getName()), ite);
                    }
                }
            };
        }
    }


    public ActionAttacher defineAction(JMenuItem pItem, String pLabel, KeyStroke pKeyStroke) {
        return defineAction(pItem, createActionListenerFor(pLabel), pKeyStroke);
    }

    public ActionAttacher defineAction(JMenuItem pItem, ActionListener pActionListener, KeyStroke pKeyStroke) {
        if (pKeyStroke != null) {
            pItem.setAccelerator(pKeyStroke);
        }

        return defineAction(pItem, pActionListener);
    }

    public ActionAttacher defineAction(AbstractButton pButton, String pLabel) {
        return defineAction(pButton, createActionListenerFor(pLabel));
    }

    public ActionAttacher defineAction(AbstractButton pButton, ActionListener pActionListener) {
        ActionListener listener = (getStatusBar() == null) ? pActionListener : getStatusBar().wrapClear(pActionListener);
        pButton.addActionListener(listener);

        return this;
    }

    public ActionAttacher defineAction(MenuItem pItem, String pLabel) {
        return defineAction(pItem, createActionListenerFor(pLabel));
    }

    public ActionAttacher defineAction(MenuItem pItem, ActionListener pActionListener) {
        ActionListener listener = (getStatusBar() == null) ? pActionListener : getStatusBar().wrapClear(pActionListener);
        pItem.addActionListener(listener);

        return this;
    }


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}