/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.*;
import java.util.Collection;
import java.util.ArrayList;

import javax.swing.*;

import org.apache.log4j.Logger;

/**
 * A small splash screen to show whenever the application wants.  The
 * subclasses can be singletons.
 *
 * @author  murali
 */
public abstract class BaseSplash extends JWindow {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(BaseSplash.class);


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private boolean fIsInitialized = false;
    private final Collection<Window> fDependentWindows = new ArrayList<Window>();
    private DependentStrategy fStrategy = DependentStrategy.VISIBLE;


    //------------------------------------------------------------
    //- Constructors

    public BaseSplash() {
        super();
    }


    //------------------------------------------------------------
    //- Accessors

    protected boolean getIsInitialized() {
        return fIsInitialized;
    }

    public DependentStrategy getStrategy() {
        return fStrategy;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setIsInitialized(boolean pIsInitialized) {
        fIsInitialized = pIsInitialized;
    }

    public void setStrategy(DependentStrategy pStrategy) {
        fStrategy = pStrategy;
    }


    //------------------------------------------------------------
    //- UI Building

    protected void centerInScreen() {
        Dimension screenSize = getToolkit().getScreenSize();
        Point midpoint = new Point(screenSize.width/2, screenSize.height/2);
        centerAt(midpoint);
    }

    protected void centerAt(Point pMidpoint) {
        Dimension windowSize = getPreferredSize();
        int x = pMidpoint.x - windowSize.width/2;
        int y = pMidpoint.y - windowSize.height/2;
        setLocation(x, y);
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void initialize() {
        getContentPane().add(makeContents());

        pack();
        centerInScreen();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void addDependent(Window pWindow) {
        fDependentWindows.add(pWindow);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void setVisible(boolean pFlag) {
        // For each dependent, make it invisible if this is visible.  And vice versa.
        for (Window dependentWindow : fDependentWindows) {
            getStrategy().handle(dependentWindow, !pFlag);
        }

        if (pFlag) {
            if (!getIsInitialized()) {
                initialize();
                setIsInitialized(true);
            }

            toFront();
        }

        super.setVisible(pFlag);
    }

    protected abstract JPanel makeContents();


    //------------------------------------------------------------
    //- Inner Classes

    public enum DependentStrategy {
        ENABLE {
            void handle(Window pWindow, boolean pState) {
                pWindow.setEnabled(pState);
            }
        },
        VISIBLE {
            void handle(Window pWindow, boolean pState) {
                pWindow.setVisible(pState);
            }};
        abstract void handle(Window pWindow, boolean pState);
    }


    //------------------------------------------------------------
    //- Main



}
