/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.Dimension;
import java.util.*;

import javax.swing.*;

import org.apache.log4j.Logger;

/**
 * A map containing all fields (UI components) in a form.
 *
 * @author  murali
 */
public class FormFieldMap {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FormFieldMap.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String,FormField> fMap = new HashMap<String,FormField>();
    private final List<FormField> fFields = new ArrayList<FormField>();


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors

    public Map<String,FormField> getMap() {
        return fMap;
    }

    public List<FormField> getFields() {
        return fFields;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public void add(String pLabel, FormField pField) {
        getMap().put(pLabel, pField);
        getFields().add(pField);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public FormFieldMap add(String pLabel, FormComponent pComponent) {
        add(pLabel, new FormField.Generic(pLabel, pComponent));

        return this;
    }

    public FormFieldMap add(String pLabel, JTextField pTextField) {
        add(pLabel, new FormField.Field(pLabel, pTextField));

        return this;
    }

    public FormFieldMap add(String pLabel, JTextArea pTextArea) {
        add(pLabel, new FormField.Area(pLabel, pTextArea));

        return this;
    }

    public FormFieldMap add(String pLabel, JEditorPane pEditorPane) {
        add(pLabel, new FormField.Html(pLabel, pEditorPane));

        return this;
    }

    public FormFieldMap add(String pLabel, JComboBox pComboBox) {
        add(pLabel, new FormField.Choice(pLabel, pComboBox));

        return this;
    }

    public FormFieldMap add(String pLabel, JCheckBox pCheckBox) {
        add(pLabel, new FormField.CheckBox(pLabel, pCheckBox));

        return this;
    }

    public FormFieldMap add(String pLabel, JList pList) {
        add(pLabel, new FormField.List(pLabel, pList));

        return this;
    }

 
    public FormFieldMap addField(String pLabel, String pText) {
        JTextField field = new JTextField(pText);
        field.setEditable(false);
        add(pLabel, field);

        return this;
    }

    public FormFieldMap addField(String pLabel, String pText, int pCols) {
        JTextField field = new JTextField(pText, pCols);
        field.setEditable(false);
        add(pLabel, field);

        return this;
    }

    public FormFieldMap addArea(String pLabel, String pText, int pRows, int pCols) {
        JTextArea area = new JTextArea(pText, pRows, pCols);
        area.setEditable(false);
        area.setLineWrap(true);
        area.setWrapStyleWord(true);

        add(pLabel, area);

        return this;
    }

    public FormFieldMap addHtml(String pLabel, String pText) {
        JEditorPane html = new JEditorPane("text/html", pText);
        html.setPreferredSize(new Dimension(25, 300));
        html.setEditable(false);

        add(pLabel, html);

        return this;
    }

    public FormFieldMap addChoice(String pLabel, Vector pChoices) {
        JComboBox choice = new JComboBox(pChoices);

        add(pLabel, choice);

        return this;
    }

    public FormFieldMap addCheckBox(String pLabel, boolean pValue) {
        JCheckBox checkBox = new JCheckBox();
        checkBox.setSelected(pValue);

        add(pLabel, checkBox);

        return this;
    }

    public FormFieldMap addList(String pLabel, Vector pChoices) {
        JList list = new JList(pChoices);

        add(pLabel, list);

        return this;
    }


    public JTextField getField(String pLabel) {
        FormField field = getMap().get(pLabel);
        return (JTextField) field.getComponent();
    }

    public JTextArea getArea(String pLabel) {
        FormField field = getMap().get(pLabel);
        return (JTextArea) field.getComponent();
    }

    public JComboBox getChoice(String pLabel) {
        FormField field = getMap().get(pLabel);
        return (JComboBox) field.getComponent();
    }

    public JCheckBox getCheckBox(String pLabel) {
        FormField field = getMap().get(pLabel);
        return (JCheckBox) field.getComponent();
    }


    public Object getValue(String pLabel) {
        Object result = null;

        FormField field = getMap().get(pLabel);
        if (field == null) {
            LOG.error("No field named '" + pLabel + "'");
        } else {
            result = field.getValue();
        }

        return result;
    }

    public void setValue(String pLabel, Object pObject) {
        FormField field = getMap().get(pLabel);
        if (field == null) {
            // This is not necessarily an error.  This particular
            // field may be hidden.  NOOP in that case.
            LOG.debug("No field named '" + pLabel + "'");
        } else {
            field.setValue(pObject);
        }
    }

    public void setHelpText(String pLabel, String pText) {
        FormField field = getMap().get(pLabel);
        if (field == null) {
            LOG.error("No field named '" + pLabel + "'");
        } else {
            field.setHelpText(pText);
        }
    }


    public JPanel toPanel() {
        FormPanelCreator creator = new FormPanelCreator(this);
        return creator.createPanel();
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
