package com.notuvy.gui;

import org.apache.log4j.Logger;

import java.awt.event.*;

import com.notuvy.thread.ThreadWorker;

// TODO: Document the threading.

/**
 * This is a facility for handling a single, simple UI action.  Very often, UI action handlers
 * only need to perform simple behavior.  For example, a key listener most often performs an action
 * on 'key released' and ignores the initial 'key pressed'.  This framework encapsulates that by
 * defining a principle action for each type of handler and stubbing the functionality of others.
 *
 * A problem with generic Listeners is that if an exception or error happens in the body,
 * it is in the UI thread.  It may get swallowed or logged to an unexpected location
 * (like System.err). Also there is no hook for handling.
 *
 * This fixes that by passing the logger and error message
 * to the constructor.  A hook method gets called in exception handling, which subclasses
 * can override with additional functionality.
 *
 * There is also an overridable method that controls whether the error message is logged or not.
 *
 * Finally, there is built-in support for performing actions in a separate thread.  This is
 * important for Swing actions.  Normally generic Listeners are executed in the Swing UI thread.
 * In most cases, the action is short and simple and this is the desired behavior.  But for long
 * running actions, it may be desirable to free the Swing thread to continue with UI redrawing
 * (for example, animating a progress bar).  To handle this, a boolean flag is passed to the
 * constructor indicating that the action is to be performed in the separate thread reserved for
 * this class.
 * Note that this should be done judiciously.  In most cases, the Swing thread is the sufficient
 * and desirable worker.
 *
 * User: murali
 * Date: May 10, 2010
 * Time: 3:56:27 PM
 */
public abstract class SimpleHandler implements Runnable, ActionListener, KeyListener, WindowListener {

    //------------------------------------------------------------
    //- Class Variables

    private static final ThreadWorker gWorker = new ThreadWorker("SimpleHandler Worker");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Logger fLogger;
    private final String fMessage;
    private final boolean fSeparateThread;
    private ActionEvent fActionEvent = null;
    private KeyEvent fKeyEvent = null;
    private WindowEvent fWindowEvent = null;

    //------------------------------------------------------------
    //- Constructors

    protected SimpleHandler(Logger pLogger, String pMessage) {
        this (pLogger, pMessage, false);
    }

    protected SimpleHandler(Logger pLogger, String pMessage, boolean pSeparateThread) {
        fLogger = pLogger;
        fMessage = pMessage;
        fSeparateThread = pSeparateThread;
    }

    //------------------------------------------------------------
    //- Getters

    public ActionEvent getActionEvent() {
        return fActionEvent;
    }

    public KeyEvent getKeyEvent() {
        return fKeyEvent;
    }

    public WindowEvent getWindowEvent() {
        return fWindowEvent;
    }


    //------------------------------------------------------------
    //- Settors

    private void setActionEvent(ActionEvent pActionEvent) {
        fActionEvent = pActionEvent;
    }

    private void setKeyEvent(KeyEvent pKeyEvent) {
        fKeyEvent = pKeyEvent;
    }

    public void setWindowEvent(WindowEvent pWindowEvent) {
        fWindowEvent = pWindowEvent;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions: ActionListener

    /**
     * The entry point from the framework.
     * It is made non-overridable to subclasses so this functionality works properly.
     * Subclasses instead implement actionWithLogging().
     *
     * @param ae  The action event which will be stored as a member variable.
     */
    public final void actionPerformed(ActionEvent ae) {
        setActionEvent(ae);

        if (fSeparateThread) {
            gWorker.enqueueAsync(this);
        } else {
            try {
                run();
            } finally {
                setActionEvent(null);
            }
        }
    }

    //------------------------------------------------------------
    //- Class Interface Functions: KeyListener

    public void keyTyped(KeyEvent pKeyEvent) {
        // NOOP
    }

    public void keyPressed(KeyEvent pKeyEvent) {
        // NOOP
    }

    public void keyReleased(KeyEvent pKeyEvent) {
        setKeyEvent(pKeyEvent);

        if (fSeparateThread) {
            gWorker.enqueueAsync(this);
        } else {
            try {
                run();
            } finally {
                setKeyEvent(null);
            }
        }
    }

    //------------------------------------------------------------
    //- Class Interface Functions: Window Listener

    public void windowOpened(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowClosed(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowIconified(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowDeiconified(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowActivated(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowDeactivated(WindowEvent pWindowEvent) {
        // NOOP
    }

    public void windowClosing(WindowEvent pWindowEvent) {
        setWindowEvent(pWindowEvent);

        if (fSeparateThread) {
            gWorker.enqueueAsync(this);
        } else {
            try {
                run();
            } finally {
                setWindowEvent(null);
            }
        }
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    /**
     * The work performed in the separate thread.
     * Note that it should catch and handle all exceptions and errors, so none of these
     * should slip through the cracks.
     */
    public final void run() {
        try {
            principleAction();
        } catch (Throwable thr) {
            if (doLogError() && (fLogger != null)) {
                fLogger.error(fMessage, thr);
            }
            handleError();
        }
    }

    /**
     * This is the single action of the class.
     * Each of the abstract subclasses of a particular handler use it as an indirection to a particular
     * method of this class.
     */
    protected abstract void principleAction();

    /**
     * A hook for additional error handling.
     * It is empty here, but subclasses can add functionality.
     */
    public void handleError() {
    }

    /**
     * By default be verbose.  Subclasses can override.
     *
     * @return  A boolean flag indicating whether to explicitly log the error to the given logger.
     */
    public boolean doLogError() {
        return true;
    }

    //------------------------------------------------------------
    //- Inner Classes

    public static abstract class Action extends SimpleHandler {
        protected Action(Logger pLogger, String pMessage) {
            super(pLogger, pMessage);
        }

        protected Action(Logger pLogger, String pMessage, boolean pSeparateThread) {
            super(pLogger, pMessage, pSeparateThread);
        }

        protected void principleAction() {
            actionPerformed();
        }

        public abstract void actionPerformed();
    }

    public static abstract class KeyRelease extends SimpleHandler {
        protected KeyRelease(Logger pLogger, String pMessage) {
            super(pLogger, pMessage);
        }

        protected KeyRelease(Logger pLogger, String pMessage, boolean pSeparateThread) {
            super(pLogger, pMessage, pSeparateThread);
        }

        protected void principleAction() {
            keyReleased();
        }

        public abstract void keyReleased();
    }

    public static abstract class WindowClosing extends SimpleHandler {
        protected WindowClosing(Logger pLogger, String pMessage) {
            super(pLogger, pMessage);
        }

        protected WindowClosing(Logger pLogger, String pMessage, boolean pSeparateThread) {
            super(pLogger, pMessage, pSeparateThread);
        }

        protected void principleAction() {
            windowClosing();
        }

        public abstract void windowClosing();
    }

    // TODO: Doubleclick (MouseListener)

    //------------------------------------------------------------
    //- Main

}
