/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import java.awt.Component;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ScheduledFuture;

import javax.swing.*;

import org.apache.log4j.Logger;

import com.notuvy.thread.NamedThreadFactory;

/**
 * An abstract dialog that is displayed only for a limited amount of time.
 *
 * A separate timeout thread is used to wait and fire after the time limit to close
 * the dialog if the user has not acted on it via the GUI.
 *
 * @author  murali
 */
public abstract class TimedDialog implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(TimedDialog.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private JDialog fDialog;
    private final Component fParent;
    private final String fTitle;
    private final String fMessage;
    private final int fTimeout;
    private boolean fClosed = false;
    private boolean fExpired = false;
    private boolean fInitialized = false;
    private final ScheduledExecutorService fService = Executors.newSingleThreadScheduledExecutor(new NamedThreadFactory("TimedDialog"));


    //------------------------------------------------------------
    //- Constructors

    protected TimedDialog(Component pParent, String pTitle, String pMessage, int pTimeout) {
        fParent = pParent;
        fTitle = pTitle;
        fMessage = pMessage;
        fTimeout = pTimeout;
    }


    //------------------------------------------------------------
    //- Accessors

    private JDialog getDialog() {
        return fDialog;
    }

    protected Component getParent() {
        return fParent;
    }

    protected String getTitle() {
        return fTitle;
    }

    protected String getMessage() {
        return fMessage;
    }

    private int getTimeout() {
        return fTimeout;
    }

    /**
     * Flag indicating whether the displayed dialog has been closed (by either the user or the timeout thread).
     *
     * @return boolean flag value.
     */
    public boolean getClosed() {
        return fClosed;
    }

    /**
     * Flag indicating whether the dialog has been displayed.
     *
     * @return boolean flag value.
     */
    public boolean isInitialized() {
        return fInitialized;
    }

    /**
     * Flag indicating whether the timeout thread acted to close the dialog.
     *
     * @return boolean flag value.
     */
    public boolean isExpired() {
        return fExpired;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void show() {
        JOptionPane pane = createOptionPane();
        fDialog = pane.createDialog(getParent(), getTitle());

        ScheduledFuture<?> timedBackgroundClose = fService.schedule(this, getTimeout(), TimeUnit.MILLISECONDS);

        fInitialized = true;
        displayDialogAndBlock();
        fClosed = true;

        if (!timedBackgroundClose.isDone()) {
            timedBackgroundClose.cancel(false);
        }
    }

    protected void displayDialogAndBlock() {
        getDialog().setVisible(true);  // This will display the UI window and suspend this thread.
    }

    protected void closeDialogAndUnblock() {
        getDialog().dispose();
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected abstract JOptionPane createOptionPane();


    public void run() {
        if (!getClosed()) {
            fExpired = true;
            closeDialogAndUnblock();
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
