/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import com.notuvy.xml.PersistentJdom;

import java.awt.Point;
import java.io.File;
import java.io.StringReader;
import java.util.*;

import org.apache.commons.lang.StringUtils;
import org.jdom.*;

/**
 * Keep track of the locations of multiple windows.  Allow this persistent
 * data to display windows in the same locations across application lifecycles.
 *
 * @author  murali
 */
public class WindowLocation extends PersistentJdom {

    //------------------------------------------------------------
    //- Class Variables

    private static final String NODE_CONTAINER = "WindowLocations";
    private static final String NODE_WINDOW = "Window";
    private static final String ATTR_NAME = "name";
    private static final String NODE_LOCATION = "Location";
    private static final String ATTR_X = "x";
    private static final String ATTR_Y = "y";
    private static final Point UNDEFINED_POINT = new Point(0, 0);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String,Point> vMap = new HashMap<String,Point>();


    //------------------------------------------------------------
    //- Constructors

    public WindowLocation(File pFile) {
        super (pFile);

        if (pFile.exists()) {
            read();
        }
    }

    public WindowLocation(String pString) {
        super (null);

        if (StringUtils.isNotBlank(pString)) {
            StringReader reader = new StringReader(pString);
            readFrom(reader);
        }
    }


    //------------------------------------------------------------
    //- Accessors



    //------------------------------------------------------------
    //- Settors

    protected Map<String,Point> getMap() {
        return vMap;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void add(String pName, Point pPoint) {
        getMap().put(pName, pPoint);

        if (getFileImage() != null) {
            save();
        }
    }


    private Point get(String pName) {
        Point result = getMap().get(pName);
        if (result == null) {
            result = UNDEFINED_POINT;
        }
        return result;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void remember(PersistentLocatable pLocatable) {
        Point point = pLocatable.giveLocatableWindow().getLocation();
        add(pLocatable.giveWindowName(), point);
    }


    public void set(PersistentLocatable pLocatable) {
        Point point = get(pLocatable.giveWindowName());
        pLocatable.giveLocatableWindow().setLocation(point);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public String rootName() {
        return NODE_CONTAINER;
    }

    protected Element toXml() {
        Element root = createRoot();

        for (String key : getMap().keySet()) {
            Point point = getMap().get(key);

            Element window = new Element(NODE_WINDOW).setAttribute(ATTR_NAME, key);
            Element location = new Element(NODE_LOCATION)
                    .setAttribute(ATTR_X, String.valueOf(point.x))
                    .setAttribute(ATTR_Y, String.valueOf(point.y));
            window.addContent(location);
            root.addContent(window);
        }

        return root;
    }


    protected void parse(Element pRoot) {
        if (isValidRoot(pRoot)) {
            //noinspection unchecked
            List<Element> windows = (List<Element>) pRoot.getChildren(NODE_WINDOW);
            for (Element window : windows) {
                String name = window.getAttributeValue(ATTR_NAME);
                int x = 0;
                int y = 0;
                Element location = window.getChild(NODE_LOCATION);
                if (location != null) {
                    x = Integer.parseInt(location.getAttributeValue(ATTR_X));
                    y = Integer.parseInt(location.getAttributeValue(ATTR_Y));
                }
                getMap().put(name, new Point(x, y));
            }
        }
    }

    @Override
    protected void reportXmlParseError(JDOMException jdome) {
        LOG.error("Problem reading persistent window location information.");
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
