/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.*;
import java.text.DecimalFormat;

import org.apache.log4j.Logger;

/**
 * A utility that takes an original image and rotates it 90 degrees either
 * clockwise or counter-clockwise.
 * 
 * @author  murali
 * @version  $Id: RotatedImage.java,v 1.1 2006/06/22 04:01:56 murali Exp $
 */
public class RotatedImage {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(RotatedImage.class);

    protected static final DecimalFormat FORMAT = new DecimalFormat("0.000");

    public static final Angle RIGHT = new Right();
    public static final Angle LEFT = new Left();


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private BufferedImage vOriginal = null;
    private final double vAngle;
    private BufferedImage vImage = null;
    private int vOrigWidth = 0;
    private int vOrigHeight = 0;
    private int vNewWidth = 0;
    private int vNewHeight = 0;


    //------------------------------------------------------------
    //- Constructors

    protected RotatedImage(BufferedImage pOriginal, Angle pType) {
        initializeOriginal(pOriginal);
        vAngle = pType.getValue();

        initialize();
    }


    //------------------------------------------------------------
    //- Accessors

    protected BufferedImage getOriginal() {
        return vOriginal;
    }

    protected double getAngle() {
        return vAngle;
    }

    protected BufferedImage getImage() {
        return vImage;
    }

    protected int getOrigWidth() {
        return vOrigWidth;
    }

    protected int getOrigHeight() {
        return vOrigHeight;
    }

    protected int getNewWidth() {
        return vNewWidth;
    }

    protected int getNewHeight() {
        return vNewHeight;
    }


    //------------------------------------------------------------
    //- Settors

    public void setImage(BufferedImage pImage) {
        vImage = pImage;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void initialize() {
        vNewHeight = getOrigWidth();
        vNewWidth = getOrigHeight();

        int deltaX = getOrigWidth()/2;
        int deltaY = getOrigHeight()/2;

        BufferedImage destination = new BufferedImage(getNewWidth(), getNewHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics2D graphics = destination.createGraphics();
        graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);

        AffineTransform at = new AffineTransform();
        {
            at.translate(deltaY, deltaX);
            at.rotate(Math.toRadians(getAngle()));
            at.translate(-deltaX, -deltaY);
        }
        graphics.drawRenderedImage(getOriginal(), at);
        graphics.dispose();
        setImage(destination);
        initializeOriginal(null);
    }


    protected void initializeOriginal(BufferedImage pOriginal) {
        vOriginal = pOriginal;

        if (pOriginal != null) {
            vOrigWidth = pOriginal.getWidth();
            vOrigHeight = pOriginal.getHeight();
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        String angle = FORMAT.format(getAngle());
        String result = "Rotated{" + getOrigWidth() + "x" + getOrigHeight() + ", " + getNewWidth() + "x" + getNewHeight() + ", " + angle + "}";
        return result;
    }


    protected void clear() {
        setImage(null);
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes

    public static abstract class Angle {
        public abstract double getValue();
    }


    public static class Right extends Angle {
        public double getValue() {
            return 90;
        }
    }

    public static class Left extends Angle {
        public double getValue() {
            return -90;
        }
    }


    //------------------------------------------------------------
    //- Main



}
