/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.*;
import java.text.DecimalFormat;

import org.apache.log4j.Logger;

/**
 * A utility to create resized image from an original image.
 * Currently it supports only shrinking, not enlarging.
 * 
 * @author  murali
 * @version  $Id: ScaledImage.java,v 1.2 2006/06/22 04:56:29 murali Exp $
 */
public class ScaledImage {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(ScaledImage.class);

    protected static final DecimalFormat FORMAT = new DecimalFormat("0.000");


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private BufferedImage vOriginal = null;
    private final int vMaxDimension;
    private BufferedImage vImage = null;
    private int vOrigWidth = 0;
    private int vOrigHeight = 0;
    private int vNewWidth = 0;
    private int vNewHeight = 0;
    private double vScale = 1.0;


    //------------------------------------------------------------
    //- Constructors

    public ScaledImage(BufferedImage pOriginal, int pMaxDimension) {
        initializeOriginal(pOriginal);
        vMaxDimension = pMaxDimension;

        initialize();
    }


    //------------------------------------------------------------
    //- Accessors

    protected BufferedImage getOriginal() {
        return vOriginal;
    }

    protected int getMaxDimension() {
        return vMaxDimension;
    }

    public BufferedImage getImage() {
        return vImage;
    }

    protected int getOrigWidth() {
        return vOrigWidth;
    }

    protected int getOrigHeight() {
        return vOrigHeight;
    }

    protected int getNewWidth() {
        return vNewWidth;
    }

    protected int getNewHeight() {
        return vNewHeight;
    }

    protected double getScale() {
        return vScale;
    }


    //------------------------------------------------------------
    //- Settors

    public void setImage(BufferedImage pImage) {
        vImage = pImage;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void initialize() {
        if ((getOrigWidth() > getMaxDimension()) || (getOrigHeight() > getMaxDimension())) {
            if (getOrigWidth() > getOrigHeight()) {
                adjustWidth();
            } else {
                adjustHeight();
            }
        } else {
            // No resize necessary.
            vNewWidth = getOrigWidth();
            vNewHeight = getOrigHeight();
        }

        BufferedImage destination = new BufferedImage(getNewWidth(), getNewHeight(), BufferedImage.TYPE_INT_RGB);
        Graphics2D graphics = destination.createGraphics();
        graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
        AffineTransform at = AffineTransform.getScaleInstance(getScale(), getScale());
        graphics.drawRenderedImage(getOriginal(), at);
        graphics.dispose();
        setImage(destination);
        initializeOriginal(null);
    }


    protected void adjustWidth() {
        vNewWidth = getMaxDimension();
        vScale = ((double) getMaxDimension()) / getOrigWidth();
        vNewHeight = (int) Math.round(getOrigHeight() * vScale);
    }

    protected void adjustHeight() {
        vNewHeight = getMaxDimension();
        vScale = ((double) getMaxDimension()) / getOrigHeight();
        vNewWidth = (int) Math.round(getOrigWidth() * vScale);
    }


    protected void initializeOriginal(BufferedImage pOriginal) {
        vOriginal = pOriginal;

        if (pOriginal != null) {
            vOrigWidth = pOriginal.getWidth();
            vOrigHeight = pOriginal.getHeight();
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        String scale = FORMAT.format(getScale());
        String result = "Scaled{" + getOrigWidth() + "x" + getOrigHeight() + ", " + getNewWidth() + "x" + getNewHeight() + ", " + scale + "}";
        return result;
    }


    public void clear() {
        setImage(null);
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

}
