/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.apache.log4j.Logger;

/**
 * A utility to read an image from a file, and save an image to a file.
 * It can be used to cache the same source image for multiple scaled images.
 * 
 * @author  murali
 * @version  $Id: SourceImage.java,v 1.1 2006/06/22 04:01:50 murali Exp murali $
 */
public class SourceImage {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(SourceImage.class);


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private final File vSource;
    private BufferedImage vLoadedImage = null;


    //------------------------------------------------------------
    //- Constructors

    public SourceImage(File pSource) {
        vSource = pSource;
    }


    public SourceImage(File pSource, BufferedImage pLoadedImage) {
        this (pSource);

        vLoadedImage = pLoadedImage;
    }

    //------------------------------------------------------------
    //- Accessors

    public File getSource() {
        return vSource;
    }


    public BufferedImage getLoadedImage() {
        return vLoadedImage;
    }


    //------------------------------------------------------------
    //- Settors

    private void setLoadedImage(BufferedImage pLoadedImage) {
        vLoadedImage = pLoadedImage;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void readImage() {
        if (getSource() == null) {
            LOG.error("No file given.");
        } else if (getSource().exists()) {
            try {
                LOG.debug("Reading image [" + getSource() + "].");
                setLoadedImage(ImageIO.read(getSource()));
            } catch (IOException ioe) {
                LOG.error("Problem reading/writing image.", ioe);
            }
        } else {
            LOG.error("No file [" + getSource() + "].");
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        String result = "Image[" + getSource().getAbsolutePath() + "]";
        return result;
    }


    public boolean isLoaded() {
        boolean result = (getLoadedImage() != null);
        return result;
    }


    public BufferedImage getImage() {
        if (!isLoaded()) {
            readImage();
        }

        return vLoadedImage;
    }


    public void save() {
        if (!isLoaded()) {
            LOG.error("No image to save.");
        } else if (getSource() == null) {
            LOG.error("No destination file given.");
        } else {
            try {
                ImageIO.write(getImage(), "jpg", getSource());
                LOG.debug("Saved [" + getSource() + "].");
                clear();  // Garbage collect the large chunk.
            } catch (IOException ioe) {
                LOG.error("Problem reading/writing image.", ioe);
            }
        }
    }


    public void clear() {
        setLoadedImage(null);
    }


    //------------------------------------------------------------
    //- Class Interface Functions




    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
