/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.image;

import java.awt.image.BufferedImage;
import java.io.File;
import java.util.*;
import java.util.regex.*;

import org.apache.log4j.Logger;

/**
 * A utility to read an image then scale/rotate it.
 *
 * @author  murali
 */
public class TransformableImage implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(TransformableImage.class);

    protected static final Pattern P_ROTATE = Pattern.compile("^-?rotate\\s*=\\s*(right|left)$", Pattern.CASE_INSENSITIVE);
    protected static final Pattern P_SCALE = Pattern.compile("^-?scale\\s*=\\s*(\\d+)$", Pattern.CASE_INSENSITIVE);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private File vSourceFile = null;
    private File vTargetFile = null;
    private BufferedImage vImage = null;
    private RotatedImage.Angle vAngle = null;
    private int vMaxDimension = 0;
    private boolean vDoSave = false;


    //------------------------------------------------------------
    //- Constructors

    public TransformableImage() {
    }

    public TransformableImage(BufferedImage pImage) {
        setImage(pImage);
    }

    public TransformableImage(SourceImage pSourceImage) {
        setImage(pSourceImage.getImage());
        setSourceFile(pSourceImage.getSource());
        setTargetFile(pSourceImage.getSource());
    }


    //------------------------------------------------------------
    //- Accessors

    public File getSourceFile() {
        return vSourceFile;
    }

    public File getTargetFile() {
        return vTargetFile;
    }

    public BufferedImage getImage() {
        return vImage;
    }

    private RotatedImage.Angle getAngle() {
        return vAngle;
    }

    public int getMaxDimension() {
        return vMaxDimension;
    }

    public boolean getDoSave() {
        return vDoSave;
    }


    //------------------------------------------------------------
    //- Settors

    public void setSourceFile(File pSourceFile) {
        vSourceFile = pSourceFile;
    }

    public void setTargetFile(File pTargetFile) {
        vTargetFile = pTargetFile;
    }

    public void setImage(BufferedImage pImage) {
        vImage = pImage;
    }

    private void setAngle(RotatedImage.Angle pAngle) {
        vAngle = pAngle;
    }

    public void setMaxDimension(int pMaxDimension) {
        vMaxDimension = pMaxDimension;
    }

    public void setDoSave(boolean val) {
        vDoSave = val;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void clear() {
        setAngle(null);
        setMaxDimension(0);
    }


    protected void loadImage() {
        if ((getImage() == null)) {
            if (getSourceFile() == null) {
                LOG.error("No source file.");
            } else {
                SourceImage source = new SourceImage(getSourceFile());
                setImage(source.getImage());
            }
        }
    }


    protected void transform() {
        boolean changed = false;

        if (getAngle() != null) {
            LOG.debug("Rotating: " + getAngle().getValue() + " degrees.");
            RotatedImage rotated = new RotatedImage(getImage(), getAngle());
            setImage(rotated.getImage());

            changed = true;
        }

        if (getMaxDimension() > 0) {
            LOG.debug("Scaling: " + getMaxDimension());
            ScaledImage scaled = new ScaledImage(getImage(), getMaxDimension());
            setImage(scaled.getImage());

            changed = true;
        }

        if (getDoSave()) {
            if (changed) {
                SourceImage target = new SourceImage(getTargetFile(), getImage());
                target.save();
                LOG.info("Saved transformed image to [" + getTargetFile().getAbsolutePath() + "].");
            } else {
                LOG.warn("No save necessary.");
            }
        } else {
            LOG.debug("No save requested.");
        }

        clear();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void rotate(Orientation pOrientation) {
        if (pOrientation.isUp()) {
            LOG.warn("No rotation necessary.");
        } else if (pOrientation.isDown()) {
            LOG.error("Invalid rotation.");
        } else if (pOrientation.isLeft()) {
            setAngle(RotatedImage.LEFT);
        } else if (pOrientation.isRight()) {
            setAngle(RotatedImage.RIGHT);
        }
    }


    public void scaleToDimension(int pDimension) {
        setMaxDimension(pDimension);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        loadImage();

        if (getImage() == null) {
            LOG.error("No image.");
        } else {
            transform();
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            TransformableImage trans = new TransformableImage();
            trans.setDoSave(true);

            for (int i = 0; i < args.length; i++) {
                String arg = args[i];
                Matcher rotate = P_ROTATE.matcher(arg);
                Matcher scale = P_SCALE.matcher(arg);
                if (rotate.find()) {
                    if (rotate.group(1).equals("right")) {
                        trans.rotate(Orientation.RIGHT);
                    } else {
                        trans.rotate(Orientation.LEFT);
                    }
                } else if (scale.find()) {
                    trans.scaleToDimension(Integer.parseInt(scale.group(1)));
                } else {
                    File file = new File(arg);
                    if (file.exists()) {
                        if (trans.getSourceFile() == null) {
                            trans.setSourceFile(file);
                            trans.setTargetFile(file);
                            LOG.info("Setting source [" + file + "].");
                        } else {
                            LOG.info("Setting target [" + file + "].");
                            trans.setTargetFile(file);
                        }
                    } else {
                        LOG.info("Setting target [" + file + "].");
                        trans.setTargetFile(file);
                    }
                }
            }

            if (args.length > 0) {
                trans.run();
            } else {
                LOG.warn("imageTransform.sh [-scale=N] [-rotate=(right|left)] <file>");
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
