/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import org.apache.log4j.Logger;
import org.jdom.Element;

import java.io.File;
import java.util.*;
import java.text.SimpleDateFormat;

import com.notuvy.xml.PersistentJdom;
import com.notuvy.file.Directory;
import com.notuvy.file.FileContent;

/**
 * Web content download with a local cache.
 *
 * User: murali
 * Date: Sep 9, 2008
 * Time: 5:19:30 PM
 */
public class CachedDownload extends PersistentJdom {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.net");

    protected static final String CACHE_SUBDIR = "download_cache";
    private static final String CACHE_FILENAME = "download_cache.xml";

    private static final int NO_EXPIRATION = -1;

    protected static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MMM-dd HH:mm:ss");

    protected static final String XML_NODE = "Cache";

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String, CachedItem> fMap = new HashMap<String,CachedItem>();
    private final Directory fItemsDir;
    private long fExpiration = NO_EXPIRATION;

    //------------------------------------------------------------
    //- Constructors

    public CachedDownload(File pParent) {
        super(new File(pParent, CACHE_FILENAME));
        fItemsDir = Directory.ensure(new File(pParent, CACHE_SUBDIR));

        if (getFileImage().exists()) {
            read();
        }

        removeOrphans();
    }

    //------------------------------------------------------------
    //- Getters

    public long getExpiration() {
        return fExpiration;
    }

    //------------------------------------------------------------
    //- Settors

    public void setExpiration(long pExpiration) {
        fExpiration = pExpiration;
    }

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void removeOrphans() {
        Set<File> referenced = new HashSet<File>();
        for (CachedItem item : fMap.values()) {
            referenced.add(item.getFile());
        }

        for (File ondisk : fItemsDir.listContents()) {
            if (!referenced.contains(ondisk)) {
                LOG.warn("Removing orphan [" + ondisk.getName() + "]");
                if (!ondisk.delete()) {
                    LOG.error(String.format("Could not delete [%s].", ondisk.getAbsolutePath()));
                }
            }
        }
    }

    private CachedItem lookupItem(String pUrl) {
        // todo: url normalization
        Date now = new Date();

        CachedItem item = fMap.get(pUrl);
        if (item == null) {
            item = new CachedItem(fItemsDir.getFile(), pUrl, now);
            LOG.debug("Downloading new. " + pUrl);
            item.download();
            fMap.put(pUrl, item);
        } else {
            // If an expiration was set and it has expired, then re-download.
            boolean redownloadNeeded = (getExpiration() > 0) && (now.getTime() >= (item.getCreated().getTime() + getExpiration()));
            if (redownloadNeeded) {
                LOG.debug("Redownloading expired.");
                item.download();
                item.setCreated(now);
            } else {
                LOG.debug(String.format("Serve cached[%s]; created[%s].", pUrl, item.getCreated()));
            }
            item.setLastAccessed(now);
        }
        save();  // At the very least, the timestamp changed.
        return item;
    }

    protected void printSummary() {
        List<CachedItem> items = listItems(CachedItem.BY_CREATION);

        StringBuilder builder = new StringBuilder("Cached Items:\n");
        for (CachedItem item : items) {
            builder.append("  [").append(DATE_FORMAT.format(item.getCreated())).append("] ").append(item.getUrlString()).append("\n");
        }

        LOG.info(builder);
    }

    protected List<CachedItem> listItems(Comparator<CachedItem> pSortOrder) {
        List<CachedItem> items = new ArrayList<CachedItem>(fMap.size());
        items.addAll(fMap.values());
        Collections.sort(items, pSortOrder);
        return items;
    }

    //------------------------------------------------------------
    //- Public Interface Functions

    public int size() {
        return fMap.size();
    }

    public synchronized File lookupFile(String pUrl) {
        CachedItem item = lookupItem(pUrl);
        return item.getFile();
    }

    public synchronized String lookupString(String pUrl) {
        CachedItem item = lookupItem(pUrl);

        FileContent fc = new FileContent(item.getFile());
        return fc.readString();
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    protected Element toXml() {
        Element result = new Element(XML_NODE);
        for (CachedItem item : fMap.values()) {
            result.addContent(item.toXml());
        }
        return result;
    }

    protected void parse(Element pRoot) {
        for (Object object : pRoot.getChildren(CachedItem.XML_NODE)) {
            CachedItem item = new CachedItem(fItemsDir.getFile());
            item.parse((Element) object);
            if (item.getFile().exists()) {
                fMap.put(item.getUrlString(), item);
            } else {
                LOG.warn("Missing content file [" + item.getFile().getName() + "]");
            }
        }
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        for (String arg : args) {
            File parent = new File(arg);
            File actual = new File(parent, CACHE_SUBDIR);
            if (actual.exists()) {
                CachedDownload cd = new CachedDownload(parent);
                cd.printSummary();
            } else {
                LOG.error("No such directory [" + actual.getAbsolutePath() + "]");
            }
        }
    }
}
