/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;
import org.jdom.Element;

import java.io.File;
import java.util.Date;
import java.util.Comparator;

import com.notuvy.xml.SimpleJdom;
import com.notuvy.util.UniqueId;

/**
 * A single cached entry.
 *
 * User: murali
 * Date: Sep 9, 2008
 * Time: 5:22:51 PM
 */
class CachedItem extends SimpleJdom {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.net");

    private static final Date UNDEFINED_DATE = new Date(0);

    protected static final String XML_NODE = "Item";
    protected static final String XML_ATTR_URL = "url";
    protected static final String XML_ATTR_CREATED = "created";
    protected static final String XML_ATTR_LASTACCESS = "lastaccess";

    protected static final Comparator<CachedItem> BY_CREATION = new Comparator<CachedItem>() {

        public int compare(CachedItem o1, CachedItem o2) {
            return o1.getCreated().compareTo(o2.getCreated());
        }
    };

    protected static final Comparator<CachedItem> BY_LAST_ACCESS = new Comparator<CachedItem>() {

        public int compare(CachedItem o1, CachedItem o2) {
            return o1.getLastAccessed().compareTo(o2.getLastAccessed());
        }
    };

    protected static final Comparator<CachedItem> BY_URL = new Comparator<CachedItem>() {

        public int compare(CachedItem o1, CachedItem o2) {
            return o1.getUrlString().compareTo(o2.getUrlString());
        }
    };

    //------------------------------------------------------------
    //- Class Functions

    private static Date toDate(String pString) {
        Date result = UNDEFINED_DATE;  // Pessimistic.
        if (StringUtils.isNotBlank(pString)) {
            long millis = Long.parseLong(pString);
            result = new Date(millis);
        }
        return result;
    }

    //------------------------------------------------------------
    //- Instance Variables

    private final File fParent;
    private String fUrlString;
    private File fFile;
    private Date fCreated;
    private Date fLastAccessed;

    //------------------------------------------------------------
    //- Constructors

    CachedItem(File pParent) {
        fParent = pParent;
    }

    public CachedItem(File pParent, String pUrlString, Date pCreated) {
        fParent = pParent;
        fUrlString = pUrlString;
        fCreated = pCreated;
        fLastAccessed = pCreated;
        fFile = new File(pParent, UniqueId.next());
    }

    //------------------------------------------------------------
    //- Getters

    public Date getCreated() {
        return fCreated;
    }

    public File getFile() {
        return fFile;
    }

    public Date getLastAccessed() {
        return fLastAccessed;
    }

    public String getUrlString() {
        return fUrlString;
    }

    //------------------------------------------------------------
    //- Settors

    protected void setCreated(Date pCreated) {
        fCreated = pCreated;
    }

    private void setFile(File pFile) {
        fFile = pFile;
    }

    protected void setLastAccessed(Date pLastAccessed) {
        fLastAccessed = pLastAccessed;
    }

    private void setUrlString(String pUrlString) {
        fUrlString = pUrlString;
    }

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public void download() {
        Download download = new Download(getUrlString(), getFile());
        download.run();
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String rootName() {
        return XML_NODE;
    }

    protected Element toXml() {
        Element result = new Element(XML_NODE);
        result.setAttribute(XML_ATTR_CREATED, String.valueOf(getCreated().getTime()));
        result.setAttribute(XML_ATTR_LASTACCESS, String.valueOf(getLastAccessed().getTime()));
        result.setAttribute(XML_ATTR_URL, getUrlString());
        result.setText(getFile().getName());

        return result;
    }

    protected void parse(Element pRoot) {
        setUrlString(pRoot.getAttributeValue(XML_ATTR_URL));
        setCreated(toDate(pRoot.getAttributeValue(XML_ATTR_CREATED)));
        setLastAccessed(toDate(pRoot.getAttributeValue(XML_ATTR_LASTACCESS)));
        setFile(new File(fParent, pRoot.getTextTrim()));
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
