/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import com.notuvy.util.Progressable;
import com.notuvy.thread.ThreadWorker;
import com.notuvy.gui.PersistentLocatable;
import com.notuvy.gui.WindowLocation;
import com.notuvy.gui.FormFieldMap;
import com.notuvy.gui.StripPanelCreator;
import com.notuvy.file.Directory;

class DownloadGui implements Runnable, Progressable, PersistentLocatable {


    //------------------------------------------------------------
    //- Class Variables

    private static final ThreadWorker LAUNCHER = new ThreadWorker("DownloadGUI");

    private static final Directory HOME = Directory.ensureInHome(".mkutils");
    public static final WindowLocation WINDOW_LOCATION = new WindowLocation(HOME.entry("dgwindows.xml"));


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Download vDownload;
    private final JDialog vDialog;
    private final JProgressBar vProgressBar = new JProgressBar();
    private final JButton vButtonCancel = new JButton("Cancel");
    private final FormFieldMap vFields = new FormFieldMap();


    //------------------------------------------------------------
    //- Constructors

    public DownloadGui(String pTitle, Download pDownload) {
        vDialog = new JDialog((Frame)null, pTitle);
        vDownload = pDownload;

        pDownload.setProgress(this);

        initializeUIComponents();
        buildUI();
    }


    //------------------------------------------------------------
    //- Accessors

    public Download getDownload() {
        return vDownload;
    }

    protected JDialog getDialog() {
        return vDialog;
    }

    protected JProgressBar getProgressBar() {
        return vProgressBar;
    }

    protected JButton getButtonCancel() {
        return vButtonCancel;
    }

    protected FormFieldMap getFields() {
        return vFields;
    }


    //------------------------------------------------------------
    //- UI Building

    protected void initializeUIComponents() {
        getDialog().setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        final PersistentLocatable pl = this;
        getButtonCancel().addActionListener(new ActionListener() {
                public void actionPerformed (ActionEvent ae) {
                    WINDOW_LOCATION.remember(pl);
                    if (getDownload().getIsSuccessful() || getDownload().getIsAborted()) {
                        System.exit(0);
                    } else {
                        getDownload().setIsAborted(true);
                        getButtonCancel().setText("Close");
                    }
                }
            });
    }

    protected void buildUI() {
        JPanel buttons = new StripPanelCreator.Horizontal()
            .add(getButtonCancel())
            .createPanel();

        Container main = getDialog().getContentPane();
        {
            main.setLayout(new BorderLayout());
            main.add(createMainPanel(), BorderLayout.CENTER);
            main.add(buttons, BorderLayout.SOUTH);
        }

        getDialog().pack();
    }


    protected JPanel createMainPanel() {
        getFields()
            .addField("From", getDownload().getUrl(), 20)
            .addField("To", getDownload().getFile().getAbsolutePath(), 20)
            .addField("Size", "");

        JPanel result = new JPanel(new BorderLayout());
        result.add(getFields().toPanel(), BorderLayout.CENTER);
        result.add(getProgressBar(), BorderLayout.SOUTH);
        return result;
    }


    //------------------------------------------------------------
    //- UI Action Handling


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void launch() {
        LAUNCHER.enqueueAsync(this);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        start();
    }


    public int getMinimum() {
        return getProgressBar().getMinimum();
    }

    public int getMaximum() {
        return getProgressBar().getMaximum();
    }

    public int getValue() {
        return getProgressBar().getValue();
    }

    public void setMinimum(int pValue) {
        getProgressBar().setMinimum(pValue);
    }

    public void setMaximum(int pValue) {
        getProgressBar().setMaximum(pValue);
        getFields().setValue("Size", String.valueOf(pValue));
    }

    public void setValue(int pValue) {
        getProgressBar().setValue(pValue);
    }

    public void incrementValue(int pValue) {
        int newValue = pValue + getProgressBar().getValue();
        setValue(newValue);
    }

    public void setString(String pString) {
        getProgressBar().setString(pString);
    }
    
    public void start() {
        WINDOW_LOCATION.set(this);
        getDialog().setVisible(true);
    }

    public void finished() {
        getButtonCancel().setText("Close");
        getButtonCancel().requestFocus();
    }

    public String giveWindowName() {
        return "downloadgui";
    }

    public Window giveLocatableWindow() {
        return getDialog();
    }
}
