/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.net;

import java.io.*;
import java.net.UnknownHostException;
import java.util.*;

import org.apache.commons.net.ftp.*;
import org.apache.log4j.Logger;

import com.notuvy.file.Directory;
import com.notuvy.util.StoredCredentials;

/**
 * A simple FTP client.
 * 
 * @author  murali
 * @version  $Id: FtpAgent.java,v 1.4 2006/06/22 03:48:21 murali Exp $
 */
public class FtpAgent implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(FtpAgent.class);

    private static FileFilter FILTER = new FileFilter() {
            public boolean accept(File pathname) {
                boolean result = pathname.isFile();
                return result;
            }
        };

    private static final Action UNDEFINED_ACTION = new Noop();


    //------------------------------------------------------------
    //- Class Functions



    //------------------------------------------------------------
    //- Instance Variables

    private String vHost = "";
    private String vUser = "";
    private String vPass = "";
    private final LinkedList vDirs = new LinkedList();
    private final FTPClient vClient = new FTPClient();
    private final Set vExistingFiles = new HashSet();
    private Action vAction = UNDEFINED_ACTION;
    private boolean vSuccess = false;
    private boolean vCreatePath = false;
    private int vCount = 0;
 


    //------------------------------------------------------------
    //- Constructors

    public FtpAgent() {
    }

    public FtpAgent(String pHost, String pUser, String pDir) {
        this ();

        initialize(pHost, pUser, pDir);
    }


    public void initialize(String pHost, String pUser, String pDir) {
        vHost = pHost;
        vUser = pUser;

        String dir = pDir.replaceFirst("(/|\\\\)$", "");
        getDirs().clear();
        String[] parts = dir.split("/");
        for (int i = 0; i < parts.length; i++) {
            if (!"".equals(parts[i])) {
                getDirs().add(parts[i]);
            }
        }
    }



    //------------------------------------------------------------
    //- Accessors

    public String getHost() {
        return vHost;
    }

    public String getUser() {
        return vUser;
    }

    public boolean getSuccess() {
        return vSuccess;
    }

    protected LinkedList getDirs() {
        return vDirs;
    }

    protected FTPClient getClient() {
        return vClient;
    }

    protected Set getExistingFiles() {
        return vExistingFiles;
    }

    protected Action getAction() {
        return vAction;
    }

    public boolean getCreatePath() {
        return vCreatePath;
    }

    public int getCount() {
        return vCount;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setSuccess(boolean pSuccess) {
        vSuccess = pSuccess;
    }

    protected void setAction(Action pAction) {
        vAction = pAction;
    }

    public void setCreatePath(boolean pCreatePath) {
        vCreatePath = pCreatePath;
    }

    private void setCount(int val) {
        vCount = val;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected String getPass() {
        if ("".equals(vPass) && !"".equals(getHost()) && !"".equals(getUser())) {
            StoredCredentials credentials = StoredCredentials.getInstance();
            vPass = credentials.giveOrCreatePassword(getHost(), getUser());
        }

        return vPass;
    }


    protected boolean chdir() {
        boolean success = true;  // Optimistic.

        String builtPath = "";
        try {
            Iterator iterator = getDirs().iterator();
            while (success && iterator.hasNext()) {
                String subdir = (String) iterator.next();
                builtPath = ("".equals(builtPath)) ?
                    subdir :
                    builtPath+ "/" + subdir;
                DirContents contents = new DirContents(getClient());
                if (!contents.exists(subdir)) {
                    if (getCreatePath()) {
                        LOG.info("Creating subdirectory [" + builtPath + "]");
                        success = getClient().makeDirectory(subdir);
                    } else {
                        LOG.error("No such path [" + givePath() + "]");
                        success = false;
                    }
                }

                if (success) {
                    if (getClient().changeWorkingDirectory(subdir)) {
                        LOG.debug("Chdir to [" + builtPath + "].");
                    } else {
                        success = false;
                        LOG.error("Problem chdir to [" + subdir + "].");
                    }
                }
            }
        } catch (IOException ioe) {
            success = false;
            if(getClient().isConnected()) {
                try {
                    LOG.error("Disconnecting.");
                    getClient().disconnect();
                } catch(IOException f) {
                    // do nothing
                }
            }
            LOG.error("Problem with chdir on server.", ioe);
        }

        return success;
    }


    protected boolean connect() {
        boolean result = false;

        try {
            getClient().connect(getHost());
            int reply = getClient().getReplyCode();
            if (FTPReply.isPositiveCompletion(reply)) {
                LOG.debug("Connected to [" + getHost() + "].");
                if (getClient().login(getUser(), getPass())) {
                    LOG.debug("Logged in as [" + getUser() + "].");
                    if (getClient().setFileType(FTP.BINARY_FILE_TYPE)) {
                        result = true;
                    } else {
                        LOG.error("Could not set file transfer type binary.");
                    }
                } else {
                    LOG.error("Error logging in as [" + getUser() + "].");
                }
            } else {
                LOG.error("Disconnecting.");
                getClient().disconnect();
                LOG.error("Bad reply from [" + getHost() + "].");
            }
        } catch (UnknownHostException uhe) {
            LOG.error("Could not connect to server; " + uhe);
        } catch (IOException ioe) {
            LOG.error("Problem with connection to server.", ioe);
        } finally {
            if(!getClient().isConnected()) {
                try {
                    LOG.error("Disconnecting.");
                    getClient().disconnect();
                } catch(IOException f) {
                    // do nothing
                }
            }
        }

        return result;
    }


    protected void recordCurrentRemoteContents() {
        DirContents contents = new DirContents(getClient());
        if (!contents.getHasError()) {
            Iterator iterator = contents.listPlainFiles().iterator();
            while (iterator.hasNext()) {
                FTPFile file = (FTPFile) iterator.next();
                getExistingFiles().add(file.getName());
            }
        }
    }


    protected void deleteUnreplacedRemoteFiles() {
        try {
            Iterator iterator = getExistingFiles().iterator();
            while (iterator.hasNext()) {
                String filename = (String) iterator.next();
                if (getClient().deleteFile(filename)) {
                    LOG.debug("Deleted: " + filename);
                } else {
                    LOG.warn("Could not delete: " + filename);
                }
            }
        } catch (IOException ioe) {
            LOG.error("Could not delete from current dir.", ioe);
        }
    }


    protected void recordCurrentLocalContents(File pLocalDir) {
        getExistingFiles().addAll(listLocalDirFiles(pLocalDir));
    }


    protected void deleteUnreplacedLocalFiles() {
        Iterator iterator = getExistingFiles().iterator();
        while (iterator.hasNext()) {
            File file = (File) iterator.next();
            if (file.delete()) {
                LOG.debug("Deleted: " + file);
            } else {
                LOG.warn("Could not delete: " + file);
            }
        }
    }


    protected void disconnect() {
        try {
            getClient().logout();
        } catch(IOException f) {
            // do nothing
        }
        try {
            LOG.debug("Disconnecting.");
            getClient().disconnect();
        } catch(IOException f) {
            // do nothing
        }
    }


    protected void put(Collection pFiles) {
        // LOG.info("Publishing " + pFiles.size() + " files by ftp.");
        setCount(pFiles.size());
        Iterator iterator = pFiles.iterator();
        while (iterator.hasNext()) {
            File local = (File) iterator.next();
            put(local, local.getName());
        }
    }


    protected void get(Collection pFiles, File pLocal) {
        // LOG.info("Fetching " + pFiles.size() + " files by ftp.");
        setCount(pFiles.size());
        Iterator iterator = pFiles.iterator();
        while (iterator.hasNext()) {
            FTPFile remote = (FTPFile) iterator.next();
            String remoteName = remote.getName();
            getExistingFiles().remove(remoteName);

            File local = new File(pLocal, remoteName);
            get(remoteName, local);
        }
    }


    protected void put(File pFile, String pRemoteName) {
        try {
            FileInputStream fis = new FileInputStream(pFile);

            if (getClient().storeFile(pRemoteName, fis)) {
                getExistingFiles().remove(pRemoteName);
                LOG.debug(String.format("Published [%s] to [%s/%s]", pFile.getAbsolutePath(), givePath(), pRemoteName));
                setSuccess(true);
            } else {
                LOG.error(String.format("Problem uploading [%s] to [%s/%s]", pFile.getAbsolutePath(), givePath(), pRemoteName));
            }

            fis.close();
        } catch (FileNotFoundException fnfe) {
            LOG.error("Could not find local file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem saving remote file.", ioe);
        }
    }


    protected void get(String pRemoteName, File pLocalFile) {
        Directory parent = new Directory(pLocalFile.getParentFile());
        // If the parent is null, then it is in cwd; do nothing for that.
        if (parent.getFile() != null) {
            parent.ensure();
        }

        String fullname = givePath() + "/" + pRemoteName;
        try {
            FileOutputStream fos = new FileOutputStream(pLocalFile);

            if (getClient().retrieveFile(pRemoteName, fos)) {
                LOG.debug("Retrieved [" + fullname + "] to [" + pLocalFile.getAbsolutePath() + "]");
                setSuccess(true);
            } else {
                LOG.error("Problem downloading [" + fullname + "]; probably does not exist.");
            }

            fos.close();
        } catch (FileNotFoundException fnfe) {
            LOG.error("Could not find local file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem saving remote file.", ioe);
        }
    }


    protected void delete(String pRemoteName) {
        try {
            String fullname = givePath() + "/" + pRemoteName;
            if (getClient().deleteFile(pRemoteName)) {
                LOG.debug("Deleted [" + fullname + "]");
                setSuccess(true);
            } else {
                LOG.error("Problem deleting [" + fullname + "]; probably does not exist.");
            }
        } catch (FileNotFoundException fnfe) {
            LOG.error("Could not find local file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem saving remote file.", ioe);
        }
    }


    protected Collection listLocalDirFiles(File pDir) {
        Collection result = new TreeSet();

        File[] array = pDir.listFiles(FILTER);
        for (int i = 0; (array != null) && (i < array.length); i++) {
            result.add(array[i]);
        }

        return result;
    }


    protected void runAction(Action pAction) {
        setAction(pAction);
        run();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        String result = getUser() + "@" + getHost() + ":" + givePath();

        return result;
    }


    public boolean isDefined() {
        boolean result = (!"".equals(getHost())) &&
            (!"".equals(getUser()));
        return result;
    }


    public String givePath() {
        StringBuffer buffer = new StringBuffer();

        boolean first = true;
        Iterator iterator = getDirs().iterator();
        while (iterator.hasNext()) {
            if (first) {
                first = false;
            } else {
                buffer.append('/');
            }
            buffer.append(iterator.next());
        }

        return buffer.toString();
    }


    public void putDir(File pDir) {
        final Collection files = listLocalDirFiles(pDir);

        if (files.isEmpty()) {
            LOG.error("Local directory is empty.");
        } else {
            Action action = new Action() {
                public void run() {
                    recordCurrentRemoteContents();
                    put(files);
                    deleteUnreplacedRemoteFiles();
                }
            };
            runAction(action);
        }
    }


    public void getDir(final File pLocalDir) {

        Action action = new Action() {
                public void run() {
                    DirContents contents = new DirContents(getClient());
                    recordCurrentLocalContents(pLocalDir);
                    get(contents.listPlainFiles(), pLocalDir);
                    deleteUnreplacedLocalFiles();
                }
            };
        runAction(action);
    }


    public void putFile(File pFile) {
        putFile(pFile, pFile.getName());
    }


    public void putFile(final File pFile, final String pRemoteName) {
        if (pFile.isDirectory()) {
            getDirs().add(pRemoteName);  // Temporarily modify the path.
            putDir(pFile);
            getDirs().removeLast();  // Restore the original values.
        } else {
            Action action = new Action() {
                    public void run() {
                        put(pFile, pRemoteName);
                    }
                };
            runAction(action);
        }
    }


    public void getFile(final String pRemoteName, final File pLocalFile) {
        if (pLocalFile.isDirectory()) {
            getDirs().add(pRemoteName);  // Temporarily modify the path.
            getDir(pLocalFile);
            getDirs().removeLast();  // Restore the original values.
        } else {
            Action action = new Action() {
                    public void run() {
                        get(pRemoteName, pLocalFile);
                    }
                };
            runAction(action);
        }
    }


    public void deleteFile(final String pRemoteName) {
        Action action = new Action() {
                public void run() {
                    delete(pRemoteName);
                }
            };
        runAction(action);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        try {
            if (isDefined()) {
                if (connect() && chdir()) {
                    getAction().run();

                    disconnect();
                }
            } else {
                LOG.error("Connection information not defined: " + this);
            }
        } catch (Throwable thr) {
            LOG.error("Problem with an FTP operation.", thr);
        }
    }


    //------------------------------------------------------------
    //- Inner Classes

    static abstract class Action implements Runnable {
    }


    static class Noop extends Action {
        public void run() {
            LOG.error("No action defined.");
        }
    }


    class DirContents {

        private final Map vMap = new HashMap();
        private boolean vHasError = false;

        DirContents(FTPClient client) {
            super();

            try {
                FTPFile[] files = client.listFiles();
                for (int i = 0; (files != null) && (i < files.length); i++) {
                    FTPFile file = files[i];
                    getMap().put(file.getName(), file);
                }
            } catch (IOException ioe) {
                vHasError = true;
                LOG.error("Problem listing dir contents.", ioe);
            }
        }


        protected Map getMap() {
            return vMap;
        }

        protected boolean getHasError() {
            return vHasError;
        }


        protected FTPFile lookup(String pName) {
            FTPFile result = (FTPFile) getMap().get(pName);
            return result;
        }


        public boolean exists(String pName) {
            boolean result = (lookup(pName) != null);
            return result;
        }


        public boolean isDirectory(String pName) {
            FTPFile file = lookup(pName);
            boolean result = (file != null) && file.isDirectory();
            return result;
        }


        public Collection listPlainFiles() {
            Collection result = new ArrayList();

            Iterator iterator = getMap().values().iterator();
            while (iterator.hasNext()) {
                FTPFile file = (FTPFile) iterator.next();
                if (file.isFile()) {
                    result.add(file);
                }
            }

            return result;
        }
    }

    //------------------------------------------------------------
    //- Main



}
