package com.notuvy.podcast;

import org.jdom.Element;
import org.apache.commons.lang.StringUtils;

import java.util.regex.Pattern;

/**
 * A single podcast.  One of potentially several items in a channel.
 *
 * @author murali
 */
public class PodcastItem extends PodcastBase {

    //------------------------------------------------------------
    //- Class Variables

    public static final String XML_Node_item = "item";
    private static final String XML_Node_title = "title";
    private static final String XML_Node_link = "link";
    private static final String XML_Node_desc = "description";
    private static final String XML_Node_duration = "duration";
    private static final String XML_Node_date = "pubDate";
    private static final String XML_Node_guid = "guid";
    private static final String XML_Node_enclosure = "enclosure";
    private static final String XML_Attr_enclosure_url = "url";
    private static final String XML_Node_enclosure_length = "length";
    private static final String XML_Node_summary = "summary";

    private static final Pattern P_just_digits = Pattern.compile("^\\d+$");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private String fDuration;
    private String fDate;
    private String fGuid;
    private String fEnclosureUrl;
    private String fEnclosureLength;
    private String fSummary = "";
    private boolean fValid = true;  // Pessimistic


    //------------------------------------------------------------
    //- Constructors

    PodcastItem(Element pElement) {
        parse(pElement);
    }

    protected PodcastItem(String pTitle, String pLink, String pDesc, String pDuration, String pDate, String pGuid, String pEnclosureUrl, String pEnclosureLength) {
        super (pTitle, pLink, pDesc);
        setDuration(pDuration);
        setDate(pDate);
        setGuid(pGuid);
        setEnclosureUrl(pEnclosureUrl);
        setEnclosureLength(pEnclosureLength);
    }

    //------------------------------------------------------------
    //- Accessors

    public String getDuration() {
        return fDuration;
    }

    public String getDate() {
        return fDate;
    }

    public String getGuid() {
        return fGuid;
    }

    public String getEnclosureUrl() {
        return fEnclosureUrl;
    }

    public String getEnclosureLength() {
        return fEnclosureLength;
    }

    public boolean isValid() {
        return fValid;
    }

    public String getSummary() {
        return fSummary;
    }


    //------------------------------------------------------------
    //- Settors

    private void setDuration(String pDuration) {
        fDuration = pDuration;
    }

    private void setDate(String pDate) {
        fDate = pDate;
    }

    private void setGuid(String pGuid) {
        fGuid = pGuid;
    }

    private void setEnclosureUrl(String pEnclosureUrl) {
        fEnclosureUrl = pEnclosureUrl;
    }

    private void setEnclosureLength(String pEnclosureLength) {
        fEnclosureLength = pEnclosureLength;
    }

    public PodcastItem setSummary(String pSummary) {
        fSummary = pSummary;

        return this;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void markInvalid(String pReason) {
        fValid = false;
        LOG.debug(pReason);
    }

    //------------------------------------------------------------
    //- Public Interface Functions


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String rootName() {
        return XML_Node_item;
    }

    protected Element toXml() {
        Element result = createRoot();
        result.addContent(new Element(XML_Node_title).setText(getTitle()));
        result.addContent(new Element(XML_Node_link).setText(getSiteUrl()));
        result.addContent(new Element(XML_Node_desc).setText(getDescription()));
        if (StringUtils.isNotBlank(getDuration())) {
            result.addContent(new Element(XML_Node_duration, gNsItunes).setText(getDuration()));
        }
        result.addContent(new Element(XML_Node_summary, gNsItunes).setText(getSummary()));
        result.addContent(new Element(XML_Node_date).setText(getDate()));
        result.addContent(new Element(XML_Node_guid).setText(getGuid()));
        result.addContent(new Element(XML_Node_enclosure).setAttribute(XML_Attr_enclosure_url, getEnclosureUrl()).setAttribute(XML_Node_enclosure_length, getEnclosureLength()));

        return result;
    }

    protected void parse(Element pRoot) {
        setTitle(getTextFromElement(pRoot.getChild(XML_Node_title)));
        setSiteUrl(getTextFromElement(pRoot.getChild(XML_Node_link)));
        setDescription(getTextFromElement(pRoot.getChild(XML_Node_desc)));
        {
            Element durationElement = pRoot.getChild(XML_Node_duration, gNsItunes);
            String durationString = getTextFromElement(durationElement);
            if (P_just_digits.matcher(durationString).find()) {
                int totalSeconds = Integer.parseInt(durationString);
                int minutes = totalSeconds / 60;
                int remainingSeconds = totalSeconds % 60;
                durationString = String.format("%d:%02d", minutes, remainingSeconds);
            }
            setDuration(durationString);
        }
        setDate(getTextFromElement(pRoot.getChild(XML_Node_date)));
        setGuid(getTextFromElement(pRoot.getChild(XML_Node_guid)));

        Element elemSummary = pRoot.getChild(XML_Node_summary, gNsItunes);
        if (elemSummary != null) {
            setSummary(getTextFromElement(elemSummary));
        }

        Element enclosure = pRoot.getChild(XML_Node_enclosure);
        if (enclosure == null) {
            markInvalid("No enclosure: "+getTitle());
        } else {
            setEnclosureUrl(enclosure.getAttributeValue(XML_Attr_enclosure_url));
            setEnclosureLength(enclosure.getAttributeValue(XML_Node_enclosure_length));
        }
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("Item");
        builder.append("[").append(getDate()).append("]");
        builder.append("[").append(getEnclosureUrl().replaceFirst("^.+/([^/]+)$", "$1")).append("]");
        return builder.toString();
    }

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}