/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.script;

import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * All the data necessary to create a script file that launces a
 * Java application.
 *
 * User: murali
 * Date: Jan 3, 2008
 * Time: 10:57:52 AM
 */
public class Script {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.script");

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private String fName;
    private String fMainJar;
    private String fMainClass;
    private String fLog4j;
    private final Collection<String> fClasspath = new ArrayList<String>();

    //------------------------------------------------------------
    //- Constructors

    public Script(Element pRoot) {
        if (pRoot.getName().equals("Script")) {
            setName(pRoot.getAttributeValue("name"));
            setMainJar(pRoot.getAttributeValue("mainjar"));
            setMainClass(pRoot.getChildText("Main"));
            setLog4j(pRoot.getChildText("Log4j"));

            //noinspection unchecked
            List<Element> classpaths = pRoot.getChildren("Classpath");
            for (Element classpath : classpaths) {
                getClasspath().add(classpath.getText());
            }
        } else {
            LOG.error("Invalid defining node: " + pRoot);
        }
    }

    //------------------------------------------------------------
    //- Getters

    public String getName() {
        return fName;
    }

    public String getMainJar() {
        return fMainJar;
    }

    public String getMainClass() {
        return fMainClass;
    }

    public String getLog4j() {
        return fLog4j;
    }

    public Collection<String> getClasspath() {
        return fClasspath;
    }

    //------------------------------------------------------------
    //- Settors

    protected void setName(String pName) {
        fName = pName;
    }

    protected void setMainJar(String pMainJar) {
        fMainJar = pMainJar;
    }

    protected void setMainClass(String pMainClass) {
        fMainClass = pMainClass;
    }

    protected void setLog4j(String pLog4j) {
        fLog4j = pLog4j;
    }

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public void generate(File pDir, Properties pProperties) {
        boolean valid = true;  // Optimistic.
        List<String> realClasspath = new ArrayList<String>(getClasspath().size());
        for (String reference : getClasspath()) {
            String real = pProperties.getProperty(reference);
            if (real == null) {
                LOG.error("Invalid reference [" + reference + "]");
                valid = false;
            } else {
                realClasspath.add(real);
            }
        }
        if (valid) {
            String basename = getName() + ".sh";
            File result = new File(pDir, basename);
            try {
                FileWriter writer = new FileWriter(result);
                writer.write("#!/bin/sh\n\n");
                writer.write("JAVA_EXE=\"" + System.getProperty("java.home") + "/bin/java\"\n\n");
                writer.write("APP_CLASSPATH=\"" + pProperties.getProperty("install.lib.dir") + "/" + getMainJar() + "\"\n");
                for (String cp : realClasspath) {
                    writer.write("APP_CLASSPATH=\"${APP_CLASSPATH}" + System.getProperty("path.separator") + cp + "\"\n");
                }
                writer.write("\nCLASS=\"" + getMainClass() + "\"\n\n");
                writer.write("LOG_CONFIG=\"-Dlog4j.configuration=" + getLog4j() + "\"\n\n");
                writer.write("${JAVA_EXE} ${JAVA_VM_ARGS} -cp ${APP_CLASSPATH} ${LOG_CONFIG} ${CLASS} $@\n");
                writer.close();
                LOG.info("Saved [" + result.getAbsolutePath() + "]");
            } catch (IOException e) {
                LOG.error("Problem writing result to [" + result.getAbsolutePath() + "]");
            }
        }
    }

    public void remove(File pDir) {
        String basename = getName() + ".sh";
        File target = new File(pDir, basename);
        if (target.delete()) {
            LOG.info("Removed [" + target.getAbsolutePath() + "]");
        }
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
