/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.script;

import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.notuvy.xml.PersistentJdom;
import com.notuvy.cmd.CommandSet;
import com.notuvy.cmd.Command;
import com.notuvy.cmd.CommandArgs;

/**
 * Generate all the script files.
 *
 * User: murali
 * Date: Jan 3, 2008
 * Time: 11:01:33 AM
 */
public class ScriptGenerator extends PersistentJdom {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger("com.notuvy.script");

    private static final String LOCAL_PROPERTIES = "local.properties";
    private static final String INSTALL_BIN_DIR = "install.bin.dir";


    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Properties fProperties = new Properties();
    private final File fInstallDir;
    private final Collection<Script> fScripts = new ArrayList<Script>();
    private final CommandSet fCommandSet = new CommandSet();


    //------------------------------------------------------------
    //- Constructors

    public ScriptGenerator() {
        super (new File("scripts.xml"));

        File installDir = null;

        File local = new File(LOCAL_PROPERTIES);
        if (local.exists()) {
            try {
                FileInputStream fis = new FileInputStream(local);
                getProperties().load(fis);
                fis.close();

                installDir = new File(getProperties().getProperty(INSTALL_BIN_DIR));
                if (installDir.exists()) {
                    // LOG.info("Installing scripts in [" + installDir + "]");
                    read();
                } else {
                    LOG.error("Invalid installation dir [" + installDir + "]");
                }
            } catch (FileNotFoundException fnfe) {
                LOG.error("problem reading file", fnfe);
            } catch (IOException ioe) {
                LOG.error("problem reading file", ioe);
            }
        } else {
            LOG.error("No such file [" + LOCAL_PROPERTIES + "]");
        }

        fInstallDir = installDir;

        getCommandSet()
                .define(new Command("generate") {
                    public boolean process(CommandArgs pArgs) {
                        boolean result = false;
                        if (pArgs.exhausted()) {
                            generate();
                            result = true;
                        }
                        return result;
                    }
                })
                .define(new Command("remove") {
                    public boolean process(CommandArgs pArgs) {
                        boolean result = false;
                        if (pArgs.exhausted()) {
                            remove();
                            result = true;
                        }
                        return result;
                    }
                });
    }

    //------------------------------------------------------------
    //- Getters

    public Properties getProperties() {
        return fProperties;
    }

    public File getInstallDir() {
        return fInstallDir;
    }

    public Collection<Script> getScripts() {
        return fScripts;
    }

    public CommandSet getCommandSet() {
        return fCommandSet;
    }

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public void generate() {
        for (Script script : getScripts()) {
            script.generate(getInstallDir(), getProperties());
        }
    }

    public void remove() {
        for (Script script : getScripts()) {
            script.remove(getInstallDir());
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected Element toXml() {
        return null;  // never need to save.
    }

    protected void parse(Element pRoot) {
        if (pRoot.getName().equals("Scripts")) {
            //noinspection unchecked
            List<Element> children = pRoot.getChildren("Script");
            // LOG.info("Found " + children.size() + " children.");
            for (Element child : children) {
                getScripts().add(new Script(child));
            }
        } else {
            LOG.error("Invalid defining node: " + pRoot);
        }
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        ScriptGenerator sg = new ScriptGenerator();
        sg.getCommandSet().process(args);
    }

}
