/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.sgml;

import java.util.List;
import java.util.ArrayList;
import java.util.Date;
import java.io.Writer;
import java.io.IOException;


/**
 * A utility to build a single tag for an SGML document, or to recurively
 * build the entire document.  Some HTML shortcuts are provided by the
 * inner classes.
 *
 * User: murali
 * Date: Aug 12, 2008
 * Time: 2:31:20 PM
 */
public class SgmlTag implements Writable {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final String fTag;
    private final List<Writable> fContents = new ArrayList<Writable>();
    private final StringBuilder fAttributes = new StringBuilder();

    //------------------------------------------------------------
    //- Constructors

    public SgmlTag(String pTag) {
        fTag = pTag;
    }

    //------------------------------------------------------------
    //- Getters


    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public SgmlTag addSubTree(SgmlTag pTag) {
        fContents.add(pTag);
        return this;
    }

    public SgmlTag addText(Object pObject) {
        fContents.add(new SgmlText(String.valueOf(pObject)));
        return this;
    }

    public SgmlTag attribute(String pKey, Object pVal) {
        fAttributes.append(" ").append(pKey).append("=\"").append(pVal).append("\"");
        return this;
    }

    public SgmlTag parent(SgmlTag pParent) {
        pParent.addSubTree(this);
        return this;
    }

    public SgmlTag newChild(String pTag) {
        SgmlTag child = new SgmlTag(pTag);
        addSubTree(child);
        return child;
    }

    public SgmlTag writeTo(StringBuilder pBuilder) {
        write(pBuilder);

        return this;
    }

    public SgmlTag writeTo(Writer pWriter) throws IOException {
        write(pWriter);

        return this;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        StringBuilder builder = new StringBuilder();
        writeTo(builder);
        return builder.toString();
    }

    public void write(StringBuilder pBuilder) {
        pBuilder.append("<").append(fTag).append(fAttributes).append(">");
        for (Writable content : fContents) {
            content.write(pBuilder);
        }
        pBuilder.append("</").append(fTag).append(">\n");
    }

    public void write(Writer pWriter) throws IOException {
        pWriter.write("<");
        pWriter.write(fTag);
        pWriter.write(fAttributes.toString());
        pWriter.write(">");
        for (Writable content : fContents) {
            content.write(pWriter);
        }
        pWriter.write("</");
        pWriter.write(fTag);
        pWriter.write(">\n");
    }


    //------------------------------------------------------------
    //- Inner Classes

    public static class A extends SgmlTag {
        private A(Object pHref) {
            super ("a");
            attribute("href", pHref);
        }

        public A(Object pHref, SgmlTag pSgmlTag) {
            this (pHref);
            addSubTree(pSgmlTag);
        }

        public A(Object pHref, String pText) {
            this (pHref);
            addText(pText);
        }
    }

    public static class Html extends SgmlTag {
        private final SgmlTag fHead;
        private final SgmlTag fBody;

        public Html(String pTitle) {
            super ("html");

            fHead = newChild("head");
            fBody = newChild("body");

            getHead().newChild("title").addText(pTitle);
        }

        public SgmlTag getHead() {
            return fHead;
        }

        public SgmlTag getBody() {
            return fBody;
        }
    }

    public static class Timestamp extends SgmlTag {
        public Timestamp() {
            this ("");
        }
        public Timestamp(String pPrefix) {
            super ("p");
            attribute("class", "timestamp");
            addText(pPrefix);
            addText(new Date());
        }
    }

    //------------------------------------------------------------
    //- Main

}
