/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.thread;

import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.log4j.Logger;
import com.notuvy.util.WallClock;

/**
 * Basically an infinite loop with a sleep period (to avoid CPU
 * saturation).  It is used to continuously execute one or more
 * given Runnables.  An instance can either be passed to a Thread
 * (it is a Runnable) or it can spawn its own Thread.  Once started,
 * it can be suspended (disabled), resumed (enabled), and stopped.
 *
 * @author  murali
 */
public class ContinuousExecution implements Runnable, ThreadControllable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ContinuousExecution.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final ThreadControl fControl = new ThreadControl();
    private long fFireTime = 0;
    private final AtomicLong fIterations = new AtomicLong();
    private int fPeriod = 1000;
    private final Collection<Runnable> fChildren = new ArrayList<Runnable>();
    private ScheduledExecutorService fService;
    private ScheduledFuture fFuture = null;


    //------------------------------------------------------------
    //- Constructors

    public ContinuousExecution(int pPeriod) {
        setPeriod(pPeriod);
    }


    //------------------------------------------------------------
    //- Accessors

    public int getPeriod() {
        return fPeriod;
    }

    protected Collection<Runnable> getChildren() {
        return fChildren;
    }

    public ThreadControl getControl() {
        return fControl;
    }

    private long getFireTime() {
        return fFireTime;
    }


    //------------------------------------------------------------
    //- Settors

    public void setPeriod(int pPeriod) {
        fPeriod = pPeriod;
    }

    private void setFireTime(long pFireTime) {
        fFireTime = pFireTime;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void runChildren() {
        for (Runnable runnable : getChildren()) {
            runnable.run();
        }
    }


    private void schedule(long pDelay) {
        fFuture = fService.schedule(this, pDelay, TimeUnit.MILLISECONDS);
    }

    private void reschedule() {
        if (getControl().isAlive()) {
            long workTime = WallClock.currentTimeMillis() - getFireTime();
            long delay = Math.max(getPeriod() - workTime, 0);  // If the work time is greater than the wait time, then fire next immediately.
            schedule(delay);
        } else {
            synchronized (this) {
                notifyAll();  // In case anyone is waiting for this to finish.
            }
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void add(Runnable pChild) {
        getChildren().add(pChild);
    }


    public void startThread(String pName) {
        startThread(pName, false);
    }

    public void startThread(String pName, boolean pRunImmediately) {
        if (fService == null) {
            fService = Executors.newSingleThreadScheduledExecutor(new NamedThreadFactory(pName));
            int delay = pRunImmediately ? 0 : getPeriod();
            schedule(delay);
        } else {
            LOG.error("Thread service was already started.  Ignoring latest attempt.");
        }
    }

    /**
     * If the thread is sleeping, force it to run now.
     */
    public void triggerNow() {
        // First check if anything was scheduled.
        if (fFuture != null) {
            // Now try to cancel (but do not interrupt an active thread).
            // If waiting work was canceled, reschedule it to go immediately
            if (fFuture.cancel(false)) {
                schedule(0);
            }
        }
    }


    @Deprecated
    /**
     * @deprecated Use getControl().suspend() instead.
     */
    public void suspend() {
        getControl().suspend();
    }

    @Deprecated
    /**
     * @deprecated Use getControl().resume() instead.
     */
    public void resume() {
        getControl().resume();
    }

    @Deprecated
    /**
     * @deprecated Use getControl().stop() instead.
     */
    public void stop() {
        getControl().stop();
    }

    public void waitForTermination() throws InterruptedException {
        synchronized (this) {
            wait();
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("ContinuousExecution[%d runnables, %d iterations]: %s", getChildren().size(), fIterations.get(), getControl().toString());
    }

    public void run() {
        setFireTime(WallClock.currentTimeMillis());

        if (getControl().isEnabled() && getControl().isAlive()) {
            fIterations.incrementAndGet();
            try {
                ThreadMonitor.update("Fired.");
                getControl().setRunning(true);
                runChildren();
            } catch (Throwable thr) {
                LOG.error("Fatal error.", thr);
            } finally {
                getControl().setRunning(false);
                ThreadMonitor.update("Finished.");
            }
        }

        reschedule();
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
