/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.thread;

import org.apache.log4j.Logger;
import com.notuvy.util.WallClock;

import java.util.concurrent.*;
import java.util.List;
import java.util.ArrayList;

/**
 * A utility to perform periodic task which are keyed off a
 * given amount of idle time.  It creates its own (daemon)
 * thread to perform the action.  It is created with the delay
 * period.  It will only trigger if the UI activity idleness is
 * greater than this given period.  It can also be triggered
 * manually, and the action will also take place in the same
 * thread.
 *
 * User: murali
 * Date: Aug 29, 2008
 * Time: 5:20:10 PM
 */
public class IdleMonitor implements Runnable, ThreadControllable {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(IdleMonitor.class);

    private static final ThreadLocal<Context> TL = new ThreadLocal<Context>() {
        @Override
        protected Context initialValue() {
            return new Context();
        }
    };


    //------------------------------------------------------------
    //- Class Functions

    public static Context getContext() {
        return TL.get();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final long fFullSleepPeriod;
    private final Runnable fRunnable;
    private final ThreadControl fControl = new ThreadControl();

    private final ManualTrigger fManual = new ManualTrigger();
    private final MonitorableList fList = new MonitorableList();

    private final ScheduledExecutorService fService;
    private ScheduledFuture<?> fFuture;


    //------------------------------------------------------------
    //- Constructors

    public IdleMonitor(final String pThreadName, long pFullSleepPeriod, Runnable pRunnable, IdlenessMonitorable...pMonitorables) {
        fService = Executors.newSingleThreadScheduledExecutor(new NamedThreadFactory(pThreadName));

        fFullSleepPeriod = pFullSleepPeriod;
        fRunnable = pRunnable;

        fList.add(fManual);
//        IdleMonitorEventQueue.register();
        fList.addAll(pMonitorables);
        // LOG.debug("First scheduled for: " + new Date(WallClock.currentTimeMillis() + pFullSleepPeriod));

        reschedule(getFullSleepPeriod());
    }

    //------------------------------------------------------------
    //- Getters

    public long getFullSleepPeriod() {
        return fFullSleepPeriod;
    }

    private Runnable getRunnable() {
        return fRunnable;
    }

    public ThreadControl getControl() {
        return fControl;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void executeRunnable() {
        if (getControl().isAlive()) {
            try {
                getControl().setRunning(true);
                getRunnable().run();

//                IdleMonitorEventQueue.touch();  // After the action is taken, mark activity.
            } catch (Throwable thr) {
                LOG.error("Fatal error with the scheduled task.", thr);
            } finally {
                getControl().setRunning(false);
                fList.touch();
            }
        }
    }

    private void reschedule(long pDelay) {
        if (getControl().isAlive()) {
            fFuture = fService.schedule(this, pDelay, TimeUnit.MILLISECONDS);
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void manualTrigger() {
        // Cancel the future trigger.
        fFuture.cancel(false);

        // Now schedule another to run immediately.
        fManual.trigger();
        reschedule(0);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        getContext().setManual(fManual.isTriggered());  // Copy the flag value beween the threads.

        long nextSleepPeriod = getFullSleepPeriod();  // Assume the full period initially.
        if (fManual.isTriggered()) {
            // ignore the last activity time and just do the action.
            executeRunnable();
        } else {
//            long lastActivity = Math.max(IdleMonitorEventQueue.getLastActivity(), fManual.lastActivity());
            long lastActivity = fList.lastActivity();
            long idlePeriod = WallClock.currentTimeMillis() - lastActivity;
            if (idlePeriod >= getFullSleepPeriod()) {
                // If idle longer than the full sleep period, then okay to do the action.
                // The next check is after the full time.
                executeRunnable();
            } else {
                // Otherwise (idle was less than the full time), sleep for the difference.
                // This way, if there still was no activity, then we wake the full time since
                // the last activity.
                nextSleepPeriod = getFullSleepPeriod() - idlePeriod;
            }
        }
        reschedule(nextSleepPeriod);
    }

    //------------------------------------------------------------
    //- Inner Classes

    public static class Context {
        private boolean fManual = false;

        public boolean isManual() {
            return fManual;
        }

        public void setManual(boolean pManual) {
            fManual = pManual;
        }
    }
    private static class MonitorableList {
        private final List<IdlenessMonitorable> fMonitorables = new ArrayList<IdlenessMonitorable>();

        public void addAll(IdlenessMonitorable[] pMonitorables) {
            for (IdlenessMonitorable monitorable : pMonitorables) {
                add(monitorable);
            }
        }

        public void add(IdlenessMonitorable monitorable) {
            if (!fMonitorables.contains(monitorable)) {
                fMonitorables.add(monitorable);
            }
        }

        public long lastActivity() {
            long latest = 0L;
            for (IdlenessMonitorable monitorable : fMonitorables) {
                latest = Math.max(latest, monitorable.lastActivity());
            }
            return latest;
        }

        public void touch() {
            for (IdlenessMonitorable monitorable : fMonitorables) {
                monitorable.markActionTakenOnIdle();
            }
        }
    }
    private static class ManualTrigger implements IdlenessMonitorable {
        private long fLastTrigger = 0L;
        private boolean fTriggered = false;

        public boolean isTriggered() {
            return fTriggered;
        }

        public long lastActivity() {
            return fLastTrigger;
        }

        public void markActionTakenOnIdle() {
            fTriggered = false;
        }

        public void trigger() {
            fTriggered = true;
            fLastTrigger = WallClock.currentTimeMillis();
        }

    }


    //------------------------------------------------------------
    //- Main

}
