package com.notuvy.thread;

import org.apache.log4j.Logger;
import org.apache.commons.lang.time.DateFormatUtils;

import java.util.*;

import com.notuvy.util.WallClock;

/**
 * Allow heartbeat monitoring of a thread.
 *
 * @author murali
 */
public class ThreadMonitor {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.util");

    private static final String FORMAT_STRING = "yyyy-MM-dd HH:mm:ss";

    private static final ThreadLocal<ThreadMonitor> TL = new ThreadLocal<ThreadMonitor>() {
        @Override
        protected ThreadMonitor initialValue() {
            return new ThreadMonitor();
        }
    };

    public static final Collection<ThreadMonitor> ALL = Collections.synchronizedList(new ArrayList<ThreadMonitor>());


    //------------------------------------------------------------
    //- Class Functions

    public static void update(String pString) {
        TL.get().updateStatus(pString);
    }

    public static void update() {
        update("Idle");
    }

    /**
     * Return the list of all monitored threads
     * @return  A clone of the full list so that list operations are thread-safe.
     */
    public static List<ThreadMonitor> allMonitoredThreads() {
        List<ThreadMonitor> copy;
        synchronized (ALL) {
            copy = new ArrayList<ThreadMonitor>(ALL);
        }
        return copy;
    }

    public static String toReport() {
        StringBuilder builder = new StringBuilder("Active Threads\n");
        for (ThreadMonitor monitor : allMonitoredThreads()) {
            builder.append(monitor).append("\n");
        }
        return builder.toString();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final Thread fThread;
    private long fLastHeartbeat = 0;
    private String fStatus = "Initial";
    private TimeZone fTimeZone = TimeZone.getDefault();

    //------------------------------------------------------------
    //- Constructors

    private ThreadMonitor() {
        fThread = Thread.currentThread();

        synchronized (ALL) {
            ALL.add(this);
        }
    }

    //------------------------------------------------------------
    //- Accessors

    public Thread getThread() {
        return fThread;
    }

    public long getLastHeartbeat() {
        return fLastHeartbeat;
    }

    public String getStatus() {
        return fStatus;
    }

    public TimeZone getTimeZone() {
        return fTimeZone;
    }


    //------------------------------------------------------------
    //- Settors

    public void setTimeZone(TimeZone pTimeZone) {
        fTimeZone = pTimeZone;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * Note that thread activity has taken place.
     *
     * @param pStatus  The string note to describe the thread activity.
     */
    public void updateStatus(String pStatus) {
        fStatus = pStatus;
        fLastHeartbeat = WallClock.currentTimeMillis();
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        String whenFired = (getLastHeartbeat() == 0) ? "Never" : DateFormatUtils.format(getLastHeartbeat(), FORMAT_STRING, getTimeZone());
        return String.format("Thread[%s] Last Fired[%s] Status[%s]", getThread().getName(), whenFired, getStatus());
    }

    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
