/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.thread;

import java.util.concurrent.*;

import org.apache.log4j.Logger;

/**
 * A separate thread engine that executes a given Runnable.
 * The purpose of this is to avoid doing "new Thread()" an arbitrary
 * number of times.  Instead, a single thread is kept and it processes
 * each give Runnable.
 *
 * @author  murali
 */
public class ThreadWorker implements ThreadControllable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ThreadWorker.class);

    private static final Future<?> DUMMY_FUTURE = new Future<Integer>() {
        public boolean cancel(boolean b) {
            return true;
        }

        public boolean isCancelled() {
            return false;
        }

        public boolean isDone() {
            return true;
        }

        public Integer get() throws InterruptedException, ExecutionException {
            return 0;
        }

        public Integer get(long l, TimeUnit pTimeUnit) throws InterruptedException, ExecutionException, TimeoutException {
            return 0;
        }
    };



    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final ThreadControl fControl = new ThreadControl();
    private final ExecutorService fService;

    //------------------------------------------------------------
    //- Constructors

    public ThreadWorker(String pName) {
        this (pName, 1);
    }

    public ThreadWorker(String pName, int pNumberThreads) {
        fService = Executors.newFixedThreadPool(pNumberThreads, new NamedThreadFactory(pName));
    }


    //------------------------------------------------------------
    //- Accessors

    public ThreadControl getControl() {
        return fControl;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public void enqueueSync(Runnable pRunnable) {
        if (getControl().isAlive()) {
            Future<?> future = enqueueAsync(pRunnable);
            try {
                future.get();
            } catch (InterruptedException ie) {
                LOG.error("Interrupted", ie);
            } catch (ExecutionException ee) {
                LOG.error("Thread execution error", ee);
            }
        } else {
            LOG.warn("Worker is shutting down.");
        }
    }

    public Future<?> enqueueAsync(final Runnable pRunnable) {
        Future<?> future = DUMMY_FUTURE;
        if (getControl().isAlive()) {
            Runnable wrapper = new Runnable() {
                public void run() {
                    try {
                        getControl().setRunning(true);
                        pRunnable.run();
                    } finally {
                        getControl().setRunning(false);
                    }
                }
            };
            future = fService.submit(wrapper);
        } else {
            LOG.warn("Worker is shutting down.");
        }
        return future;
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
