package com.notuvy.util;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.jar.Attributes;
import java.util.Properties;
import java.io.*;
import java.net.URL;

/**
 * Basic description
 *
 * @author murali
 */
public abstract class ApplicationProperties {

    // Regardless if which method is used to extract properties, it is
    // necessary to define the property values in the expected way.
    //
    // Some of the values are automatically available in the maven
    // environment.  But some of them need to be defined.
    //
    // The copyright data need to be defined as explicit property settings:
    //
    //   <project>
    //     <properties>
    //       <copyright.start>2011</copyright.start>
    //       <copyright.holder>John Doe</copyright.holder>
    //     </properties>
    //   </project>
    //
    // The build number needs a plugin to compute it:
    //
    //   <project>
    //     <build>
    //       <plugin>
    //         <groupId>org.codehaus.mojo</groupId>
    //         <artifactId>buildnumber-maven-plugin</artifactId>
    //         <version>1.0-beta-2</version>
    //         <executions>
    //           <execution>
    //             <phase>validate</phase>
    //             <goals>
    //               <goal>create</goal>
    //             </goals>
    //           </execution>
    //         </executions>
    //         <configuration>
    //           <format>{0,date,yyMMdd-HHmm}</format>
    //           <items>
    //             <item>timestamp</item>
    //           </items>
    //         </configuration>
    //       </plugin>
    //      <build>
    //   </project>

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.notuvy.util");

    public static final String UNDEFINED = "undefined";

    private static final Pattern P_FORMAT = Pattern.compile("^(\\d{2})");
    protected static final FixedWidthText LINE_BREAKER = new FixedWidthText(80);

    private static final String RESOURCE_FILE_URI = "/application_properties.xml";
    private static final String RESOURCE_MANIFEST_URI = "/META-INF/MANIFEST.MF";

    private static final String PROP_PROJECT_NAME = "ProjectName";
    private static final String PROP_PROJECT_VERSION = "ProjectVersion";
    private static final String PROP_BUILD_TIME = "BuildTime";
    private static final String PROP_PROJECT_DESCRIPTION = "ProjectDescription";
    private static final String PROP_COPYRIGHT_START = "CopyrightStart";
    private static final String PROP_COPYRIGHT_HOLDER = "CopyrightHolder";


    //------------------------------------------------------------
    //- Class Functions

    public static ApplicationProperties fromManifest(Class pClass) {
        return new Manifest(pClass);
    }

    public static ApplicationProperties fromResource(Class pClass) {
        return new Resource(pClass);
    }

    public static ApplicationProperties from(Class pClass) {
        return (pClass.getResource(RESOURCE_FILE_URI) == null) ? fromManifest(pClass) : fromResource(pClass);
    }


    //------------------------------------------------------------
    //- Instance Variables

    protected final Translation<String> fMap = new Translation<String>()
            .addTransform(Translation.CASE_INSENSITIVE)
            .setFallback(UNDEFINED);

    protected String fLicense = "";


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Gettors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void define(Object o, Object pValue) {
        String value = (String) pValue;
        if (StringUtils.isNotBlank(value)) {
            String key = String.valueOf(o);
            String val = PROP_PROJECT_DESCRIPTION.equals(key) ? LINE_BREAKER.process(value) : value;
            fMap.define(key, val);
        }
    }

    protected void readLicense(InputStream pIs) throws IOException {
        StringBuilder builder = new StringBuilder();
        InputStreamReader isr = new InputStreamReader(pIs);
        BufferedReader br = new BufferedReader(isr);

        for (String line=br.readLine(); line!=null; line=br.readLine()) {
            builder.append(line).append("\n");
        }

        br.close();
        isr.close();
        pIs.close();

        fLicense = builder.toString();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String getProjectName() {
        return fMap.lookup(PROP_PROJECT_NAME);
    }

    public String getProjectVersion() {
        return fMap.lookup(PROP_PROJECT_VERSION);
    }

    public String getBuildTime() {
        return fMap.lookup(PROP_BUILD_TIME);
    }

    public String getProjectDescription() {
        return fMap.lookup(PROP_PROJECT_DESCRIPTION);
    }

    public String getLicense() {
        return fLicense;
    }

    public String toCopyrightString() {
        StringBuilder builder = new StringBuilder("Copyright (c) ");
        String copyrightStart = fMap.lookup(PROP_COPYRIGHT_START);
        builder.append(copyrightStart);
        String buildTime = getBuildTime();
        if (StringUtils.isNotBlank(buildTime)) {
            Matcher matcher = P_FORMAT.matcher(buildTime);
            if (matcher.find()) {
                String copyrightEnd = "20" + matcher.group(1);
                if (!copyrightEnd.equals(copyrightStart)) {
                    builder.append("-").append(copyrightEnd);
                }
            }
        }
        builder.append(" ").append(fMap.lookup(PROP_COPYRIGHT_HOLDER)).append(". All rights reserved.");
        return builder.toString();
    }

    public String toProjectSummaryString() {
        StringBuilder builder = new StringBuilder();
        builder.append(getProjectName())
                .append(", version ").append(getProjectVersion())
                .append(" [").append(getBuildTime()).append("]");
        return builder.toString();
    }

    public String toAboutString() {
        StringBuilder builder = new StringBuilder();

        builder.append(toProjectSummaryString()).append("\n\n");
        builder.append(getProjectDescription()).append("\n\n");
        builder.append(toCopyrightString()).append("\n\n");
        builder.append(getLicense());

        return builder.toString();
    }

    public String toString() {
        return toProjectSummaryString();
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes

    public static class Manifest extends ApplicationProperties {

        // It is necessary to use the proper plugin to set the property values
        // in the jar manifest:
        //
        //   <project>
        //     <build>
        //       <plugin>
        //         <groupId>org.apache.maven.plugins</groupId>
        //         <artifactId>maven-jar-plugin</artifactId>
        //         <version>2.2</version>
        //         <configuration>
        //           <archive>
        //             <manifestEntries>
        //               <ProjectName>${project.name}</ProjectName>
        //               <ProjectVersion>${project.version}</ProjectVersion>
        //               <BuildTime>${buildNumber}</BuildTime>
        //               <ProjectDescription>${project.description}</ProjectDescription>
        //               <CopyrightStart>${copyright.start}</CopyrightStart>
        //               <CopyrightHolder>${copyright.holder}</CopyrightHolder>
        //             </manifestEntries>
        //           </archive>
        //         </configuration>
        //       </plugin>
        //     </build>
        //   </project>

        /**
         *
         * @param pReferenceClass  A class from the desired jar (since there could be multiple jars on the classpath, and thus multiple manifests).
         */
        public Manifest(Class pReferenceClass) {
            String classContainer = pReferenceClass.getProtectionDomain().getCodeSource().getLocation().toString();
            try {
                URL manifestUrl = new URL("jar:" + classContainer + "!" + RESOURCE_MANIFEST_URI);
                java.util.jar.Manifest manifest = new java.util.jar.Manifest(manifestUrl.openStream());
                Attributes mainAttributes = manifest.getMainAttributes();
                for (Object o : mainAttributes.keySet()) {
                    define(o, mainAttributes.get(o));
                }
            } catch (IOException ioe) {
                LOG.error("Problem reading properties.", ioe);
            }

            try {
                URL licenseUrl = new URL("jar:" + classContainer + "!/LICENSE.txt");
                InputStream is = licenseUrl.openStream();
                readLicense(is);
            } catch (FileNotFoundException fnfe) {
                LOG.error(fnfe);
            } catch (IOException ioe) {
                LOG.error("Problem reading LICENSE.txt", ioe);
            }
        }

    }

    public static class Resource extends ApplicationProperties {

        // To create a resource file in the application classpath, create the file
        // named "application_properties.xml" (RESOURCE_FILE_URI) with the content
        // below, in [src/main/filtered-resources].  Then add the following to the
        // project POM:
        //
        //   <build>
        //     <resources>
        //       <resource>
        //         <directory>src/main/resources</directory>
        //         <filtering>false</filtering>
        //       </resource>
        //       <resource>
        //         <directory>src/main/filtered-resources</directory>
        //         <filtering>true</filtering>
        //       </resource>
        //     </resources>
        //   </build>
        //
        // The file content:
        //
        // <?xml version="1.0" encoding="UTF-8" standalone="no"?>
        // <!DOCTYPE properties SYSTEM "http://java.sun.com/dtd/properties.dtd">
        // <properties>
        //   <entry key="ProjectName">${project.name}</entry>
        //   <entry key="ProjectVersion">${project.version}</entry>
        //   <entry key="BuildTime">${buildNumber}</entry>
        //   <entry key="ProjectDescription">${project.description}</entry>
        //   <entry key="CopyrightStart">${copyright.start}</entry>
        //   <entry key="CopyrightHolder">${copyright.holder}</entry>
        // </properties>

        /**
         *
         * @param pReferenceClass  A class from the desired jar (since there could be multiple jars on the classpath).
         */
        public Resource(Class pReferenceClass) {
            Properties properties = new Properties();
            try {
                InputStream is = pReferenceClass.getResourceAsStream(RESOURCE_FILE_URI);
                properties.loadFromXML(is);
                for (Object o : properties.keySet()) {
                    define(o, properties.get(o));
                }
            } catch (IOException ioe) {
                LOG.error("Problem reading properties.", ioe);
            }

            try {
                InputStream is = pReferenceClass.getResourceAsStream("/LICENSE.txt");
                readLicense(is);
            } catch (FileNotFoundException fnfe) {
                LOG.error(fnfe);
            } catch (IOException ioe) {
                LOG.error("Problem reading LICENSE.txt", ioe);
            }
        }

    }

    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            ApplicationProperties mp = ApplicationProperties.from(ApplicationProperties.class);
            LOG.info(mp);
            LOG.info(mp.toCopyrightString());
            LOG.info(mp.getProjectDescription());
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }

}
