package com.notuvy.util;

import java.text.DecimalFormat;

/**
 * Representation of a numeric real value in the range [0, 1].
 * If a value ouside this range is given, either 0 or 1 will be used.
 *
 * User: murali
 * Date: Apr 16, 2009
 * Time: 6:18:00 PM
 */
public abstract class Fractional {

    //------------------------------------------------------------
    //- Class Variables

    private static final DecimalFormat FORMAT_RAW = new DecimalFormat("0.0###");
    private static final DecimalFormat FORMAT_PERCENT = new DecimalFormat("0.0");

    //------------------------------------------------------------
    //- Class Functions

    public static Fractional valueOf(double pValue) {
        return new Raw(pValue);
    }

    public static Fractional valueOf(float pValue) {
        return new Raw(pValue);
    }

    public static Fractional ratio(int pNumerator, int pDenominator) {
        return new Rational(pNumerator, pDenominator);
    }

    public static Fractional ratio(long pNumerator, long pDenominator) {
        return new Rational(pNumerator, pDenominator);
    }

    private static double divide(long pNumerator, long pDenominator) {
        long numerator = Math.max(pNumerator, 0);
        long denominator = Math.max(pDenominator, 0);
        double result = 0.0;
        if (denominator > 0) {
            if (numerator >= denominator) {
                result = 1.0;
            } else {
                result = ((double) numerator) / denominator;
            }
        }
        return result;
    }

    //------------------------------------------------------------
    //- Instance Variables

    private final double fValue;

    //------------------------------------------------------------
    //- Constructors

    protected Fractional(double pValue) {
        fValue = pValue;
    }

    //------------------------------------------------------------
    //- Getters

    public double getValue() {
        return fValue;
    }

    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * Because "== 0" is unreliable for floating point values, we have
     * this method to do the analogous check.
     *
     * @return true if equivalent to zero (i.e. a sufficiently small value).
     */
    public boolean isZero() {
        return getValue() < 0.000001;
    }

    public String toPercentage() {
        return String.format("%s%%", FORMAT_PERCENT.format(getValue()*100));
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return FORMAT_RAW.format(getValue());
    }

    //------------------------------------------------------------
    //- Inner Classes

    static class Raw extends Fractional {

        Raw(double pValue) {
            super (Math.min(Math.max(pValue, 0.0), 1.0));
        }

    }


    static class Rational extends Fractional {

        Rational(long pNumerator, long pDenominator) {
            super (divide(pNumerator, pDenominator));
        }

    }

    //------------------------------------------------------------
    //- Main

}
