/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;


import org.jdom.Element;

import java.io.*;
import java.util.List;
import java.util.HashMap;
import java.util.Map;

import com.notuvy.xml.PersistentJdom;

/**
 * A utility to represent all the application configuration settings
 * as an XML document.
 *
 * User: murali
 * Date: Dec 30, 2007
 * Time: 7:56:40 AM
 */
public class GenericConfigurationFile extends PersistentJdom {

    //------------------------------------------------------------
    //- Class Variables

    private static final String XML_NODE_ROOT = "ApplicationConfiguration";
    private static final String XML_NODE_SETTING = "Setting";
    private static final String XML_ATTR_NAME = "name";

    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Map<String,Field> fFields = new HashMap<String,Field>();

    //------------------------------------------------------------
    //- Constructors

    public GenericConfigurationFile(File pFileImage) {
        super (pFileImage);
    }

    //------------------------------------------------------------
    //- Accessors

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    public GenericConfigurationFile add(Field pField) {
        fFields.put(pField.getName(), pField);
        return this;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    final public String rootName() {
        return XML_NODE_ROOT;
    }

    final protected Element toXml() {
        Element result = new Element(XML_NODE_ROOT);
        for (Field field : fFields.values()) {
            Element setting = new Element(XML_NODE_SETTING);
            setting.setAttribute(XML_ATTR_NAME, field.getName());
            field.setContentIn(setting);
            result.addContent(setting);
        }
        return result;

    }

    final protected void parse(Element pRoot) {
        @SuppressWarnings({"unchecked"})
        List<Element> children = (List<Element>) pRoot.getChildren(XML_NODE_SETTING);
        for (Element child : children) {
            String name = child.getAttributeValue(XML_ATTR_NAME);
            Field field = fFields.get(name);
            if (field == null) {
                LOG.warn("No handler for setting [" + name + "]");
            } else {
                field.handle(child);
            }
        }
    }

    //------------------------------------------------------------
    //- Inner Classes

    public static abstract class Field {
        private final String fName;

        protected Field(String pName) {
            fName = pName;
        }

        public String getName() {
            return fName;
        }

        protected abstract void handle(Element pElement);
        protected abstract void setContentIn(Element pElement);
    }

    public static abstract class TextField extends Field {
        protected TextField(String pName) {
            super (pName);
        }

        protected void handle(Element pElement) {
            handleSet(pElement.getText());
        }

        protected void setContentIn(Element pElement) {
            pElement.setText(handleGet());
        }

        protected abstract void handleSet(String pInput);
        protected abstract String handleGet();
    }

    /**
     * Process text and store the value locally in a ValueHolder.  Client logic simply interacts
     * with the ValueHolder.  It can be initialized with the default value.
     */
    public static class TextValueField extends TextField {
        private final ValueHolder<String> fValue;

        public TextValueField(String pName, String pInitialValue) {
            this (pName, new ValueHolder<String>(pInitialValue));
        }

        public TextValueField(String pName, ValueHolder<String> pValue) {
            super (pName);

            fValue = pValue;
        }

        public ValueHolder<String> getValue() {
            return fValue;
        }

        protected void handleSet(String pInput) {
            getValue().set(pInput);
        }

        protected String handleGet() {
            return getValue().get();
        }
    }


    //------------------------------------------------------------
    //- Main

}
