/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.lang.reflect.Method;
import java.io.IOException;

import org.apache.log4j.Logger;

/**
 * OS specific logic.
 *
 * @author  murali
 */
public abstract class OSLogic {

    //------------------------------------------------------------
    //- Class Variables

    private static final String[] UNIX_BROWSERS = {
        "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"
    };

    protected static final Logger LOG = Logger.getLogger(OSLogic.class);

    public static OSLogic INSTANCE = null;


    static {
        String osName = System.getProperty("os.name");

         if (osName.startsWith("Mac OS")) {
             INSTANCE = new Mac();
         } else if (osName.startsWith("Windows")) {
             INSTANCE = new Windows();
         } else { //assume Unix or Linux
             INSTANCE = new Unix();
         }

         LOG.info("Created " + INSTANCE);
    }


    //------------------------------------------------------------
    //- Class Functions

    public static void launchBrowser(String pUrl) {
        try {
            // Remove all whitespace first.
            INSTANCE.browse(pUrl.replaceAll("\\s+", ""));
        } catch (Exception ex) {
            LOG.error("Problem launching browser.", ex);
        }
    }


    //------------------------------------------------------------
    //- Instance Variables


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public boolean isMac() {
        return false;
    }

    public boolean isWindows() {
        return false;
    }

    public boolean isUnix() {
        return false;
    }


    protected abstract void browse(String pUrl) throws Exception;


    //------------------------------------------------------------
    //- Inner Classes

    static class Mac extends OSLogic {

        public boolean isMac() {
            return true;
        }

        protected void browse(String pUrl) throws Exception {
            Class fileMgr = Class.forName("com.apple.eio.FileManager");
            Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[] {String.class});
            openURL.invoke(null, new Object[] {pUrl});
        }

        public String toString() {
            return "Mac";
        }
    }


    static class Windows extends OSLogic {

        public boolean isWindows() {
            return true;
        }

        protected void browse(String pUrl) throws Exception {
            Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + pUrl);
        }


        public String toString() {
            return "Windows";
        }
    }


    static class Unix extends OSLogic {

        private final String vBrowser;


        protected Unix() {
            super ();

            String candidate = null;
            for (int count = 0; count < UNIX_BROWSERS.length && candidate == null; count++) {
                try {
                    if (Runtime.getRuntime().exec(new String[] {"which", UNIX_BROWSERS[count]}).waitFor() == 0) {
                        candidate = UNIX_BROWSERS[count];
                    }
                } catch (InterruptedException ie) {
                    LOG.warn("Interrupted which-ing", ie);
                } catch (IOException ioe) {
                    LOG.error("Problem which-ing", ioe);
                }
            }

            if (candidate == null) {
                vBrowser = null;
               LOG.error("Could not find web browser");
            } else {
                vBrowser = candidate;
            }
        }


        protected String getBrowser() {
            return vBrowser;
        }


        public boolean isUnix() {
            return true;
        }

        protected void browse(String pUrl) throws Exception {
            Runtime.getRuntime().exec(new String[] {getBrowser(), pUrl});
        }


        public String toString() {
            return "Unix [" + getBrowser() + "]";
        }
    }


    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            OSLogic.launchBrowser("http://www.yahoo.com");
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
