package com.notuvy.util;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

import java.util.regex.Pattern;
import java.util.regex.Matcher;


/**
 * A tool to transform strings to plain text.
 * This is done by translating non-ascii character to ascii equivalents,
 * and by translating entity characters.
 *
 * Because the result may be unsed in HTML, the entity translation can be
 * skipped.  In some cases, the non-ASCII character is first converted into
 * its HTML entity equivalent (in case that is the intended use).  It may
 * then be translated from and entity to it's ASCII equivalent (potentially
 * multiple characters).
 *
 * A full table of unicode values for characters can be found at
 * http://en.wikipedia.org/wiki/List_of_XML_and_HTML_character_entity_references
 * http://www.ssec.wisc.edu/~tomw/java/unicode.html
 *
 * User: murali
 * Date: Apr 27, 2010
 * Time: 9:54:16 AM
 */
public class PlainTextFilter {

    //------------------------------------------------------------
    //- Class Variables

    private static final Logger LOG = Logger.getLogger(PlainTextFilter.class);

    private static final Translation<String> FIX_NONASCII = new Translation.Rewrite()
            .define("\u0026", "&amp;")
            .define("\u0027", "&apos;")
            .define("\u003C", "&lt;")
            .define("\u003E", "&gt;")
            .define("\u00A0", "&nbsp;")
            .define("\u00A1", "&iexcl;")
            .define("\u00A2", "&cent;")
            .define("\u00A3", "&pound;")
            .define("\u00A4", "&curren;")
            .define("\u00A5", "&yen;")
            .define("\u00A6", "&brvbar;")
            .define("\u00A7", "&sect;")
            .define("\u00A8", "&uml;")
            .define("\u00A9", "&copy;")
            .define("\u00AA", "&ordf;")
            .define("\u00AB", "&laquo;")
            .define("\u00AC", "&not;")
            .define("\u00AD", "&shy;")
            .define("\u00AE", "&reg;")
            .define("\u00AF", "&macr;")
            .define("\u00B0", "&deg;")
            .define("\u00B1", "&plusmn;")
            .define("\u00B2", "&sup2;")
            .define("\u00B3", "&sup3;")
            .define("\u00B4", "&acute;")
            .define("\u00B5", "&micro;")
            .define("\u00B6", "&para;")
            .define("\u00B7", "&middot;")
            .define("\u00B8", "&cedil;")
            .define("\u00B9", "&sup1;")
            .define("\u00BA", "&ordm;")
            .define("\u00BB", "&raquo;")
            .define("\u00BC", "&frac14;")
            .define("\u00BD", "&frac12;")
            .define("\u00BE", "&frac34;")
            .define("\u00BF", "&iquest;")
            .define("\u00C0", "&Agrave;")
            .define("\u00C1", "&Aacute;")
            .define("\u00C2", "&Acirc;")
            .define("\u00C3", "&Atilde;")
            .define("\u00C4", "&Auml;")
            .define("\u00C5", "&Aring;")
            .define("\u00C6", "&AElig;")
            .define("\u00C7", "&Ccedil;")
            .define("\u00C8", "&Egrave;")
            .define("\u00C9", "&Eacute;")
            .define("\u00CA", "&Ecirc;")
            .define("\u00CB", "&Euml;")
            .define("\u00CC", "&Igrave;")
            .define("\u00CD", "&Iacute;")
            .define("\u00CE", "&Icirc;")
            .define("\u00CF", "&Iuml;")
            .define("\u00D0", "&ETH;")
            .define("\u00D1", "&Ntilde;")
            .define("\u00D2", "&Ograve;")
            .define("\u00D3", "&Oacute;")
            .define("\u00D4", "&Ocirc;")
            .define("\u00D5", "&Otilde;")
            .define("\u00D6", "&Ouml;")
            .define("\u00D7", "&times;")
            .define("\u00D8", "&Oslash;")
            .define("\u00D9", "&Ugrave;")
            .define("\u00DA", "&Uacute;")
            .define("\u00DB", "&Ucirc;")
            .define("\u00DC", "&Uuml;")
            .define("\u00DD", "&Yacute;")
            .define("\u00DE", "&THORN;")
            .define("\u00DF", "&szlig;")
            .define("\u00E0", "&agrave;")
            .define("\u00E1", "&aacute;")
            .define("\u00E2", "&acirc;")
            .define("\u00E3", "&atilde;")
            .define("\u00E4", "&auml;")
            .define("\u00E5", "&aring;")
            .define("\u00E6", "&aelig;")
            .define("\u00E7", "&ccedil;")
            .define("\u00E8", "&egrave;")
            .define("\u00E9", "&eacute;")
            .define("\u00EA", "&ecirc;")
            .define("\u00EB", "&euml;")
            .define("\u00EC", "&igrave;")
            .define("\u00ED", "&iacute;")
            .define("\u00EE", "&icirc;")
            .define("\u00EF", "&iuml;")
            .define("\u00F0", "&eth;")
            .define("\u00F1", "&ntilde;")
            .define("\u00F2", "&ograve;")
            .define("\u00F3", "&oacute;")
            .define("\u00F4", "&ocirc;")
            .define("\u00F5", "&otilde;")
            .define("\u00F6", "&ouml;")
            .define("\u00F7", "&divide;")
            .define("\u00F8", "&oslash;")
            .define("\u00F9", "&ugrave;")
            .define("\u00FA", "&uacute;")
            .define("\u00FB", "&ucirc;")
            .define("\u00FC", "&uuml;")
            .define("\u00FD", "&yacute;")
            .define("\u00FE", "&thorn;")
            .define("\u00FF", "&yuml;")
            .define("\u0152", "&OElig;")
            .define("\u0153", "&oelig;")
            .define("\u0160", "&Scaron;")
            .define("\u0161", "&scaron;")
            .define("\u0178", "&Yuml;")
            .define("\u0192", "&fnof;")
            .define("\u02C6", "&circ;")
            .define("\u02DC", "&tilde;")
            .define("\u0391", "&Alpha;")
            .define("\u0392", "&Beta;")
            .define("\u0393", "&Gamma;")
            .define("\u0394", "&Delta;")
            .define("\u0395", "&Epsilon;")
            .define("\u0396", "&Zeta;")
            .define("\u0397", "&Eta;")
            .define("\u0398", "&Theta;")
            .define("\u0399", "&Iota;")
            .define("\u039A", "&Kappa;")
            .define("\u039B", "&Lambda;")
            .define("\u039C", "&Mu;")
            .define("\u039D", "&Nu;")
            .define("\u039E", "&Xi;")
            .define("\u039F", "&Omicron;")
            .define("\u03A0", "&Pi;")
            .define("\u03A1", "&Rho;")
            .define("\u03A3", "&Sigma;")
            .define("\u03A4", "&Tau;")
            .define("\u03A5", "&Upsilon;")
            .define("\u03A6", "&Phi;")
            .define("\u03A7", "&Chi;")
            .define("\u03A8", "&Psi;")
            .define("\u03A9", "&Omega;")
            .define("\u03B1", "&alpha;")
            .define("\u03B2", "&beta;")
            .define("\u03B3", "&gamma;")
            .define("\u03B4", "&delta;")
            .define("\u03B5", "&epsilon;")
            .define("\u03B6", "&zeta;")
            .define("\u03B7", "&eta;")
            .define("\u03B8", "&theta;")
            .define("\u03B9", "&iota;")
            .define("\u03BA", "&kappa;")
            .define("\u03BB", "&lambda;")
            .define("\u03BC", "&mu;")
            .define("\u03BD", "&nu;")
            .define("\u03BE", "&xi;")
            .define("\u03BF", "&omicron;")
            .define("\u03C0", "&pi;")
            .define("\u03C1", "&rho;")
            .define("\u03C2", "&sigmaf;")
            .define("\u03C3", "&sigma;")
            .define("\u03C4", "&tau;")
            .define("\u03C5", "&upsilon;")
            .define("\u03C6", "&phi;")
            .define("\u03C7", "&chi;")
            .define("\u03C8", "&psi;")
            .define("\u03C9", "&omega;")
            .define("\u03D1", "&thetasym;")
            .define("\u03D2", "&upsih;")
            .define("\u03D6", "&piv;")
            .define("\u2002", "&ensp;")
            .define("\u2003", "&emsp;")
            .define("\u2009", "&thinsp;")
            .define("\u200C", "&zwnj;")
            .define("\u200D", "&zwj;")
            .define("\u200E", "&lrm;")
            .define("\u200F", "&rlm;")
            .define("\u2013", "&ndash;")
            .define("\u2014", "&mdash;")
            .define("\u2018", "&lsquo;")
            .define("\u2019", "&rsquo;")
            .define("\u201A", "&sbquo;")
            .define("\u201C", "&ldquo;")
            .define("\u201D", "&rdquo;")
            .define("\u201E", "&bdquo;")
            .define("\u2020", "&dagger;")
            .define("\u2021", "&Dagger;")
            .define("\u2022", "&bull;")
            .define("\u2026", "&hellip;")
            .define("\u2030", "&permil;")
            .define("\u2032", "&prime;")
            .define("\u2033", "&Prime;")
            .define("\u2039", "&lsaquo;")
            .define("\u203A", "&rsaquo;")
            .define("\u203E", "&oline;")
            .define("\u2044", "&frasl;")
            .define("\u20AC", "&euro;")
            .define("\u2111", "&image;")
            .define("\u2118", "&weierp;")
            .define("\u211C", "&real;")
            .define("\u2122", "&trade;")
            .define("\u2135", "&alefsym;")
            .define("\u2190", "&larr;")
            .define("\u2191", "&uarr;")
            .define("\u2192", "&rarr;")
            .define("\u2193", "&darr;")
            .define("\u2194", "&harr;")
            .define("\u21B5", "&crarr;")
            .define("\u21D0", "&lArr;")
            .define("\u21D1", "&uArr;")
            .define("\u21D2", "&rArr;")
            .define("\u21D3", "&dArr;")
            .define("\u21D4", "&hArr;")
            .define("\u2200", "&forall;")
            .define("\u2202", "&part;")
            .define("\u2203", "&exist;")
            .define("\u2205", "&empty;")
            .define("\u2207", "&nabla;")
            .define("\u2208", "&isin;")
            .define("\u2209", "&notin;")
            .define("\u220B", "&ni;")
            .define("\u220F", "&prod;")
            .define("\u2211", "&sum;")
            .define("\u2212", "&minus;")
            .define("\u2217", "&lowast;")
            .define("\u221A", "&radic;")
            .define("\u221D", "&prop;")
            .define("\u221E", "&infin;")
            .define("\u2220", "&ang;")
            .define("\u2227", "&and;")
            .define("\u2228", "&or;")
            .define("\u2229", "&cap;")
            .define("\u222A", "&cup;")
            .define("\u222B", "&int;")
            .define("\u2234", "&there4;")
            .define("\u223C", "&sim;")
            .define("\u2245", "&cong;")
            .define("\u2248", "&asymp;")
            .define("\u2260", "&ne;")
            .define("\u2261", "&equiv;")
            .define("\u2264", "&le;")
            .define("\u2265", "&ge;")
            .define("\u2282", "&sub;")
            .define("\u2283", "&sup;")
            .define("\u2284", "&nsub;")
            .define("\u2286", "&sube;")
            .define("\u2287", "&supe;")
            .define("\u2295", "&oplus;")
            .define("\u2297", "&otimes;")
            .define("\u22A5", "&perp;")
            .define("\u22C5", "&sdot;")
            .define("\u2308", "&lceil;")
            .define("\u2309", "&rceil;")
            .define("\u230A", "&lfloor;")
            .define("\u230B", "&rfloor;")
            .define("\u27E8", "&lang;")
            .define("\u27E9", "&rang;")
            .define("\u25CA", "&loz;")
            .define("\u2660", "&spades;")
            .define("\u2663", "&clubs;")
            .define("\u2665", "&hearts;")
            .define("\u2666", "&diams;")
            ;
//    .define("\u0022", "&quot;")

    private static final Translation<String> FIX_ENTITY = new Translation.Rewrite()
            .define("&quot;", "\"")
            .define("&amp;", "&")
            .define("&apos;", "'")
            .define("&lt;", "<")
            .define("&gt;", ">")
            .define("&nbsp;", " ")
            .define("&copy;", "(c)")
            .define("&laquo;", "<<")
            .define("&reg;", "(r)")
            .define("&acute;", "`")
            .define("&raquo;", ">>")
            .define("&frac14;", "1/4")
            .define("&frac12;", "1/2")
            .define("&frac34;", "3/4")
            .define("&Agrave;", "A")
            .define("&Aacute;", "A")
            .define("&Acirc;", "A")
            .define("&Atilde;", "A")
            .define("&Auml;", "A")
            .define("&Aring;", "A")
            .define("&AElig;", "AE")
            .define("&Ccedil;", "C")
            .define("&Egrave;", "E")
            .define("&Eacute;", "E")
            .define("&Ecirc;", "E")
            .define("&Euml;", "E")
            .define("&Igrave;", "I")
            .define("&Iacute;", "I")
            .define("&Icirc;", "I")
            .define("&Iuml;", "I")
            .define("&Ntilde;", "n")
            .define("&Ograve;", "N")
            .define("&Oacute;", "O")
            .define("&Ocirc;", "O")
            .define("&Otilde;", "O")
            .define("&Ouml;", "O")
            .define("&times;", "x")
            .define("&Oslash;", "O")
            .define("&Ugrave;", "U")
            .define("&Uacute;", "U")
            .define("&Ucirc;", "U")
            .define("&Uuml;", "U")
            .define("&Yacute;", "Y")
            .define("&agrave;", "a")
            .define("&aacute;", "a")
            .define("&acirc;", "a")
            .define("&atilde;", "a")
            .define("&auml;", "a")
            .define("&aring;", "a")
            .define("&aelig;", "ae")
            .define("&ccedil;", "c")
            .define("&egrave;", "e")
            .define("&eacute;", "e")
            .define("&ecirc;", "e")
            .define("&euml;", "e")
            .define("&igrave;", "i")
            .define("&iacute;", "i")
            .define("&icirc;", "i")
            .define("&iuml;", "i")
            .define("&ntilde;", "n")
            .define("&ograve;", "o")
            .define("&oacute;", "o")
            .define("&ocirc;", "o")
            .define("&otilde;", "o")
            .define("&ouml;", "o")
            .define("&divide;", "/")
            .define("&oslash;", "o")
            .define("&ugrave;", "u")
            .define("&uacute;", "u")
            .define("&ucirc;", "u")
            .define("&uuml;", "u")
            .define("&yacute;", "y")
            .define("&yuml;", "y")
            .define("&OElig;", "OE")
            .define("&oelig;", "oe")
            .define("&Scaron;", "S")
            .define("&scaron;", "s")
            .define("&Yuml;", "Y")
            .define("&tilde;", "~")
            .define("&ndash;", "---")
            .define("&mdash;", "--")
            .define("&lsquo;", "'")
            .define("&rsquo;", "'")
            .define("&ldquo;", "\"")
            .define("&rdquo;", "\"")
            .define("&hellip;", "...")
            .define("&lsaquo;", "<")
            .define("&rsaquo;", ">")
            .define("&frasl;", "/")
            .define("&real;", "R")
            .define("&trade;", "(tm)")

            .define("&#039;", "'")
            ;

    private static final Pattern P_NONASCII = Pattern.compile("([^\\p{ASCII}])");
    private static final Pattern P_ENTITY = Pattern.compile("(&(#\\d+|\\w+);)");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private boolean fDoEntity = true;


    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Getters

    public boolean isDoEntity() {
        return fDoEntity;
    }

    //------------------------------------------------------------
    //- Settors

    public PlainTextFilter setDoEntity(boolean pDoEntity) {
        fDoEntity = pDoEntity;
        return this;
    }

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private String fix(String pInput, Pattern pPattern, Translation<String> pTranslation) {
        StringBuffer result = new StringBuffer();
        Matcher matcher = pPattern.matcher(pInput);
        while (matcher.find()) {
            String offender = matcher.group(1);
            String replace = pTranslation.lookup(offender);
            //noinspection StringEquality
            if (replace == offender) {
                LOG.warn("No replacement for [" + offender + "]");
            }
            matcher.appendReplacement(result, replace);
        }
        matcher.appendTail(result);
        return result.toString();
    }
//    private String fix(String pInput, Pattern pPattern, Translation<String> pTranslation) {
//        String result = pInput;
//        Matcher matcher = pPattern.matcher(pInput);
//        while (matcher.find()) {
//            String offender = matcher.group(1);
//            String replace = pTranslation.lookup(offender);
//            //noinspection StringEquality
//            if (replace == offender) {
//                LOG.warn("No replacement for [" + offender + "]");
//            } else {
//                result = matcher.replaceFirst(replace);
//                matcher = pPattern.matcher(result);  // Search for the next.
//            }
//        }
//        return result;
//    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String fix(String pInput) {
        String result = pInput;

        if (StringUtils.isBlank(result)) {
            result = "";
        } else {
            result = fix(result, P_NONASCII, FIX_NONASCII);

            if (isDoEntity()) {
                result = fix(result, P_ENTITY, FIX_ENTITY);
            }
        }

        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main

}
