/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.io.IOException;
import java.io.StringWriter;
import java.util.*;

import org.apache.log4j.Logger;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;


/**
 * A utlity to launch an external appliction
 *
 * @author  murali
 */
public class ProgramLauncher implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ProgramLauncher.class);


    //------------------------------------------------------------
    //- Class Functions

    // We cannot simple break the string at the spaces.  If the spaces
    // are inside quotes, then they should not be broken.
    protected static String[] createCmdArray(String cmd) {
        ArrayList<String> list = new ArrayList<String>();

        boolean inDoubleQuotes = false;
        boolean inSingleQuotes = false;
        StringBuffer buffer = new StringBuffer();
        for (char ch : cmd.toCharArray()) {
            if (inDoubleQuotes) {
                if (ch == '\"') {
                    inDoubleQuotes = false;
                } else {
                    buffer.append(ch);
                }
            } else if (inSingleQuotes) {
                if (ch == '\'') {
                    inSingleQuotes = false;
                } else {
                    buffer.append(ch);
                }
            } else {
                if (ch == '\"') {
                    inDoubleQuotes = true;
                } else if (ch == '\'') {
                    inSingleQuotes = true;
                } else if (Character.isWhitespace(ch)) {
                    if (buffer.length() > 0) {
                        list.add(buffer.toString());
                        buffer.delete(0, buffer.length());
                    }
                } else {
                    buffer.append(ch);
                }
            }
        }

        if (buffer.length() > 0) {
            list.add(buffer.toString());
        }

        return list.toArray(new String[list.size()]);
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final String vCommand;
    private boolean fBlocking = false;


    //------------------------------------------------------------
    //- Constructors

    public ProgramLauncher(String pCommand) {
        vCommand = pCommand;
    }


    //------------------------------------------------------------
    //- Accessors

    public String getCommand() {
        return vCommand;
    }

    public boolean isBlocking() {
        return fBlocking;
    }


    //------------------------------------------------------------
    //- Settors

    public void setBlocking(boolean pBlocking) {
        fBlocking = pBlocking;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        return "ProgramLauncher[" + getCommand() + "]";
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        LOG.debug(this);
        String[] parts = createCmdArray(getCommand());
            Runtime runtime = Runtime.getRuntime();
            try {
                Process process = runtime.exec(parts);
                if (isBlocking()) {
                    process.waitFor();

                    StringWriter out = new StringWriter();
                    IOUtils.copy(process.getInputStream(), out);
                    String stdout = out.toString().replaceFirst("\\s+$", "");
                    if (StringUtils.isNotBlank(stdout)) {
                        LOG.info("Process stdout:\n" + stdout);
                    }

                    StringWriter err = new StringWriter();
                    IOUtils.copy(process.getErrorStream(), err);
                    String stderr = err.toString().replaceFirst("\\s+$", "");
                    if (StringUtils.isNotBlank(stderr)) {
                        LOG.error("Process stderr:\n" + stderr);
                    }
                }
            } catch (IOException ioe) {
                LOG.error(String.format("Could not exec [%s]", getCommand()), ioe);
            } catch (InterruptedException ie) {
                LOG.error(String.format("Interrupted [%s]", getCommand()), ie);
            }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
