/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.util;

import java.util.Set;
import java.util.Map;
import java.util.TreeMap;
import java.util.Collection;


/**
 * Mainly present a SortedSet, but keep a count of how many times each
 * entry was added.  The underlying Collection uses equality check to
 * determine existence
 *
 * User: murali
 */
public class SortedCountingSet<T> {

    //------------------------------------------------------------
    //- Class Variables


    //------------------------------------------------------------
    //- Class Functions

    //------------------------------------------------------------
    //- Instance Variables

    private final Map<T,Counter> fMap = new TreeMap<T,Counter>();

    //------------------------------------------------------------
    //- Constructors

    //------------------------------------------------------------
    //- Getters

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    //------------------------------------------------------------
    //- Public Interface Functions

    /**
     * Add the given element to the collection.  If it previously existed, increment its
     * counter.  Otherwise create it.
     *
     * @param pKey  The entity to add.
     */
    public void add(T pKey) {
        expect(pKey);
        fMap.get(pKey).increment();
    }


    /**
     * Initialize and element.  Add it to the set, but leave the count at 0.
     *
     * @param pKey  The element to initialize.
     */
    public void expect(T pKey) {
        if (fMap.get(pKey) == null) {
            fMap.put(pKey, new Counter());
        }
    }


    /**
     * Initialize with the given collection of elements.
     *
     * @param pKeys  A generic collection of elements.
     */
    public void expectAll(Collection<T> pKeys) {
        for (T key : pKeys) {
            expect(key);
        }
    }

    /**
     * Give the complete set of entities in this collection.
     *
     * @return  The sorted set of entities.
     */
    public Set<T> keySet() {
        return fMap.keySet();
    }

    /**
     * Give the number of times the given entity exists in the collection.
     * @param pKey  The entity to check.
     * @return  Integer count, 0 if the entity does not exist.
     */
    public long countFor(T pKey) {
        Counter counter = fMap.get(pKey);
        return (counter == null) ? 0 : counter.getValue();
    }

    /**
     * Remove the given entity  if it exists in the collection.
     * @param pKey  The entity to remove.
     * @return  Integer count, 0 if the entity does not exist.
     */
    public long remove(T pKey) {
        Counter counter = fMap.remove(pKey);
        return (counter == null) ? 0 : counter.getValue();
    }

    /**
     * The total number of (unique) entities in the collection.
     *
     * @return  Integer size.
     */
    public int size() {
        return fMap.size();
    }

    /**
     * The weighted count of all entities.
     * 
     * @return  Integer total.
     */
    public int total() {
        int result = 0;
        for (Counter counter : fMap.values()) {
            result += counter.getValue();
        }
        return result;
    }

    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return fMap.toString();
    }

    //------------------------------------------------------------
    //- Inner Classes

    //------------------------------------------------------------
    //- Main

}
