/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.util;

import java.io.*;
import java.util.*;

import javax.swing.*;

import org.apache.log4j.Logger;
import com.notuvy.file.Directory;

/**
 * Web FTP credentials.
 *
 * @author  murali
 * @version  $Id: StoredCredentials.java,v 1.4 2006/06/18 17:21:27 murali Exp $
 */
public class StoredCredentials {

    //------------------------------------------------------------
    //- Class Variables

    private static final Directory HOME = Directory.ensureInHome(".mkutils");
    public static final String PROPS_FILENAME = "stcred.properties";

    private static final Logger LOG = Logger.getLogger(StoredCredentials.class);

    private static StoredCredentials sInstance = new StoredCredentials();


    //------------------------------------------------------------
    //- Class Functions

    public static StoredCredentials getInstance() {
        return sInstance;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private Properties vProperties = new Properties();


    //------------------------------------------------------------
    //- Constructors

    private StoredCredentials() {
        initialize();
    }

    private final void initialize() {
        File file = HOME.entry(PROPS_FILENAME);
 
        if (file.exists()) {
            try {
                FileInputStream fis = new FileInputStream(file);
                getProperties().load(fis);
                fis.close();
            } catch (IOException ioe) {
                LOG.error("Could not load properties file [" + file.getAbsolutePath() + "].", ioe);
            }
        }
    }


    //------------------------------------------------------------
    //- Accessors

    protected Properties getProperties() {
        return vProperties;
    }



    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected String createKey(String pDomain, String pRef) {
        String result = pDomain + "|" + pRef;
        return result;
    }


    protected String add(String pDomain, String pRef) {
        String result = add(createKey(pDomain, pRef));
        return result;
    }


    protected String add(String pKey) {
        HashCrypt crypt = new HashCrypt(pKey);
        String entered = "";

        JPasswordField field = new JPasswordField(10);
        int action = JOptionPane.showConfirmDialog(null, field,"Password [" + pKey + "]",JOptionPane.OK_CANCEL_OPTION);
        if(action >= 0) {
            entered = new String(field.getPassword());
        }

        if ("".equals(entered)) {
            LOG.warn("No entry added.");
        } else {
            String passwd = crypt.encrypt(entered);
            getProperties().put(pKey, passwd);
            save();
        }

        return entered;
    }


    private void save() {
        File file = HOME.entry(PROPS_FILENAME);
 
        try {
            FileOutputStream fos = new FileOutputStream(file);
            getProperties().store(fos, null);
            fos.close();
        } catch (IOException ioe) {
            LOG.error("Could not save properties file [" + file + "].", ioe);
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("StoredCredentials{\n");

        TreeSet sorted = new TreeSet(getProperties().keySet());
        Iterator iterator = sorted.iterator();
        while (iterator.hasNext()) {
            buffer.append("  ").append(iterator.next()).append('\n');
        }
        buffer.append('}');

        return buffer.toString();
    }


    public String givePassword(String pDomain, String pRef) {
        String result = "";  // Pessimistic.

        String key = createKey(pDomain, pRef);
        String encrypted = getProperties().getProperty(key);
        if (encrypted != null) {
            HashCrypt crypt = new HashCrypt(key);
            result = crypt.decrypt(encrypted);
        }

        return result;
    }


    public String giveOrCreatePassword(String pDomain, String pRef) {
        String result = givePassword(pDomain, pRef);
        if ("".equals(result)) {
            LOG.debug("Adding new password for [" + pDomain + "," + pRef + "]");
            result = add(pDomain, pRef);
        }

        return result;
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            StoredCredentials cred = getInstance();

            if (args.length == 0) {
                LOG.info(cred);
            } else if (args.length == 1) {
                String key = args[0];
                LOG.info("need to add[" + key + "]");
                cred.add(key);
                LOG.info(cred);
                System.exit(0);
            } else if (args.length == 2) {
                String domain = args[0];
                String ref = args[1];
                LOG.info("need to add[" + domain + "|" + ref + "]");
                cred.add(domain, ref);
                LOG.info(cred);
                System.exit(0);
            } else {
                LOG.warn("Usage:  StoredCredentials [key]");
            }
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
