/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import org.apache.log4j.Logger;

import java.util.Date;

/**
 * System clock based unique ID generator.  The ID is an 8-character
 * alphanumeric string.
 *
 * Each time it is called, it will return a guaranteed different value.
 * Because it is system clock based, and because the result is uniformly
 * an 8-character string, the guarantee only applies over the time period
 * [01-Jan-2010, 25-May-2099].
 *
 * After that period, the logic still works.  There will be unique values
 * for the new epoch, but there is a potential reuse of values from the
 * prior epoch.
 *
 * Note that sequential values almost always differ by multiple characters
 * (and often by ALL characters).  Because of the encoding, it is not
 * apparent how far apart in time two different values were generated.
 *
 * Note that the result is case-INsensitive.  (This is meaningful really
 * only for the reverse computation of coded string to system time.)
 *
 *
 * @author  murali
 */
public class UniqueId {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(UniqueId.class);

    private static UniqueId INSTANCE = new UniqueId();

    protected static final long BASE_VALUE = 1262332800000L;
    private static final long MAX_VALUE = 2821109907455L;

    /**
     * The digits for a base-36 number.
     */
    private static char[] HASH = {
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
        'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
    };

    private static int[] UNHASH = new int[128];
    static {
        int i = 0;
        for (char ch : HASH) {
            UNHASH[(int)ch] = i++;
        }
    }


    //------------------------------------------------------------
    //- Class Functions

    public static String next() {
        return String.valueOf(INSTANCE.nextHash());
    }


    protected static void scramble(char[] buffer) {
        if (buffer.length > 0) {
            int ref = UNHASH[(int)buffer[0]];
            for (int i=1; i<buffer.length; i++) {
                int diff = UNHASH[(int)buffer[i]] - ref;
                if (diff < 0) {
                    diff += HASH.length;
                }
                buffer[i] = HASH[diff];
            }
        }
    }

    protected static void unscramble(char[] buffer) {
        if (buffer.length > 0) {
            int ref = UNHASH[(int)buffer[0]];
            for (int i=1; i<buffer.length; i++) {
                int diff = UNHASH[(int)buffer[i]] + ref;
                if (diff >= HASH.length) {
                    diff -= HASH.length;
                }
                buffer[i] = HASH[diff];
            }
        }
    }

    protected static char[] toHash(long value) {
        char[] buffer = new char[8];
        int i = 0;
        while (i < buffer.length) {
            buffer[i++] = '0';
        }
        i = 0;  // reset

        value = value % MAX_VALUE;  // handle overflow.
        while (value > 0) {
            int remainder = (int) (value % HASH.length);
            buffer[i++] = HASH[remainder];
            value = value / HASH.length;
        }

        return buffer;
    }


    protected static long fromHash(char[] buffer) {
        long result = 0;
        if (buffer.length > 0) {
            for (int i=buffer.length-1; i>=0; i--) {
                result = HASH.length * result + UNHASH[(int)buffer[i]];
            }
        }

        return result;
    }

    public static long toTime(String pHash) {
        long result = 0;
        if (pHash != null) {
            char[] buffer = pHash.toLowerCase().toCharArray();
            unscramble(buffer);
            result = fromHash(buffer) + BASE_VALUE;
        }
        return result;
    }


    //------------------------------------------------------------
    //- Instance Variables

    private long fLast = 0;

    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors

    protected long getLast() {
        return fLast;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setLast(long pLast) {
        fLast = pLast;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private synchronized long nextLong() {
        return WallClock.currentTimeMillis() - BASE_VALUE;
    }

    public synchronized char[] nextHashArray() {
        long value = nextLong();
        while (value == getLast()) {
            try {
                Thread.sleep(1);
            } catch (InterruptedException ie) {
                // nothing.
            }
            value = nextLong();
        }
        setLast(value);

        return toHash(value);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String nextHash() {
        char[] buffer = nextHashArray();
        scramble(buffer);

        return new String(buffer);
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    private static void info(long pValue) {
        Date date = new Date(pValue);
        char[] buffer = toHash(pValue - BASE_VALUE);
        String hash = new String(buffer);
        LOG.info(String.format("[%d][%s][%s][%d]", pValue, date, hash, toTime(hash)));
    }

    public static void main(String[] args) {
        try {
            long now = WallClock.currentTimeMillis();
            info(now);

            LOG.info(String.format("time [%s][%d][%s]", "xwq902v3", toTime("xwq902v3"), new Date(toTime("xwq902v3"))));
            LOG.info(String.format("time [%s][%d][%s]", "zuo7y0t1", toTime("zuo7y0t1"), new Date(toTime("zuo7y0t1"))));
            LOG.info(String.format("time [%s][%d][%s]", "1tm5wyrz", toTime("1tm5wyrz"), new Date(toTime("1tm5wyrz"))));
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }



}
